@@ -24,6 +24,9 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #include "callback.h"
 #include "access.h"
@@ -109,51 +112,51 @@ static const char *const psz_paranoia_list_text[] = { N_("none"), N_("overlap"),
  * Module descriptor
  *****************************************************************************/
 
-vlc_module_begin();
-    add_usage_hint( N_("cddax://[device-or-file][@[T]track]") );
-    set_description( N_("Compact Disc Digital Audio (CD-DA) input") );
-    set_capability( "access", 10 /* compare with priority of cdda */ );
-    set_shortname( N_("Audio Compact Disc"));
-    set_callbacks( CDDAOpen, CDDAClose );
-    add_shortcut( "cddax" );
-    add_shortcut( "cd" );
-    set_category( CAT_INPUT );
-    set_subcategory( SUBCAT_INPUT_ACCESS );
+vlc_module_begin ()
+    add_usage_hint( N_("cddax://[device-or-file][@[T]track]") )
+    set_description( N_("Compact Disc Digital Audio (CD-DA) input") )
+    set_capability( "access", 10 /* compare with priority of cdda */ )
+    set_shortname( N_("Audio Compact Disc"))
+    set_callbacks( CDDAOpen, CDDAClose )
+    add_shortcut( "cddax" )
+    add_shortcut( "cd" )
+    set_category( CAT_INPUT )
+    set_subcategory( SUBCAT_INPUT_ACCESS )
 
     /* Configuration options */
     add_integer ( MODULE_STRING "-debug", 0, CDDADebugCB,
                   N_("Additional debug"),
-                  DEBUG_LONGTEXT, true );
+                  DEBUG_LONGTEXT, true )
 
     add_integer( MODULE_STRING "-caching",
                  DEFAULT_PTS_DELAY / MILLISECONDS_PER_SEC, NULL,
                  N_("Caching value in microseconds"),
-                 CACHING_LONGTEXT, true );
+                 CACHING_LONGTEXT, true )
 
     add_integer( MODULE_STRING "-blocks-per-read",
                  DEFAULT_BLOCKS_PER_READ, CDDABlocksPerReadCB,
                  N_("Number of blocks per CD read"),
-                 BLOCKS_PER_READ_LONGTEXT, true );
+                 BLOCKS_PER_READ_LONGTEXT, true )
 
     add_string( MODULE_STRING "-title-format",
                 "Track %T. %t", NULL,
                 N_("Format to use in playlist \"title\" field when no CDDB"),
-                TITLE_FMT_LONGTEXT, true );
+                TITLE_FMT_LONGTEXT, true )
 
 #if LIBCDIO_VERSION_NUM >= 73
     add_bool( MODULE_STRING "-analog-output", false, NULL,
               N_("Use CD audio controls and output?"),
               N_("If set, audio controls and audio jack output are used"),
-              false );
+              false )
 #endif
 
     add_bool( MODULE_STRING "-cdtext-enabled", true, CDTextEnabledCB,
               N_("Do CD-Text lookups?"),
               N_("If set, get CD-Text information"),
-              false );
+              false )
 
     add_bool( MODULE_STRING "-navigation-mode", true,
-#if FIXED
+#ifdef FIXED
           CDDANavModeCB,
 #else
           NULL,
@@ -161,71 +164,71 @@ vlc_module_begin();
               N_("Use Navigation-style playback?"),
               N_("Tracks are navigated via Navagation rather than "
          "a playlist entries"),
-              false );
+              false )
 
 #if LIBCDIO_VERSION_NUM >= 72
       add_string( MODULE_STRING "-paranoia", NULL, NULL,
         PARANOIA_TEXT,
         PARANOIA_LONGTEXT,
-        false );
+        false )
       change_string_list( psz_paranoia_list, psz_paranoia_list_text, 0 );
 #endif /* LIBCDIO_VERSION_NUM >= 72 */
 
 #ifdef HAVE_LIBCDDB
-    set_section( N_("CDDB" ), 0 );
+    set_section( N_("CDDB" ), 0 )
     add_string( MODULE_STRING "-cddb-title-format",
                 "Track %T. %t - %p %A", NULL,
                 N_("Format to use in playlist \"title\" field when using CDDB"),
-                CDDB_TITLE_FMT_LONGTEXT, true );
+                CDDB_TITLE_FMT_LONGTEXT, true )
 
     add_bool( MODULE_STRING "-cddb-enabled", true, CDDBEnabledCB,
               N_("CDDB lookups"),
               N_("If set, lookup CD-DA track information using the CDDB "
                  "protocol"),
-              false );
+              false )
 
     add_string( MODULE_STRING "-cddb-server", "freedb.freedb.org", NULL,
                 N_("CDDB server"),
                 N_( "Contact this CDDB server look up CD-DA information"),
-        true );
+        true )
 
     add_integer( MODULE_STRING "-cddb-port", 8880, NULL,
                  N_("CDDB server port"),
                  N_("CDDB server uses this port number to communicate on"),
-                 true );
+                 true )
 
     add_string( MODULE_STRING "-cddb-email", "me@home", NULL,
                 N_("email address reported to CDDB server"),
                 N_("email address reported to CDDB server"),
-        true );
+        true )
 
     add_bool( MODULE_STRING "-cddb-enable-cache", true, NULL,
               N_("Cache CDDB lookups?"),
               N_("If set cache CDDB information about this CD"),
-              false );
+              false )
 
     add_bool( MODULE_STRING "-cddb-httpd", false, NULL,
               N_("Contact CDDB via the HTTP protocol?"),
               N_("If set, the CDDB server gets information via the CDDB HTTP "
                  "protocol"),
-              true );
+              true )
 
     add_integer( MODULE_STRING "-cddb-timeout", 10, NULL,
                  N_("CDDB server timeout"),
                  N_("Time (in seconds) to wait for a response from the "
                     "CDDB server"),
-                 false );
+                 false )
 
     add_string( MODULE_STRING "-cddb-cachedir", "~/.cddbslave", NULL,
                 N_("Directory to cache CDDB requests"),
                 N_("Directory to cache CDDB requests"),
-        true );
+        true )
 
     add_bool( MODULE_STRING "-cdtext-prefer", true, CDTextPreferCB,
               N_("Prefer CD-Text info to CDDB info?"),
               N_("If set, CD-Text information will be preferred "
          "to CDDB information when both are available"),
-              false );
+              false )
 #endif /*HAVE_LIBCDDB*/
 
-vlc_module_end();
+vlc_module_end ()
