@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2001-2006 the VideoLAN team
  * Copyright © 2006 Rémi Denis-Courmont
- * $Id: 79cf667b14c24daa7c782e3cd710f4ba19e3dd27 $
+ * $Id: b3dc2c911bb1a7aa9a1438c121d03c7ac9641a84 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr> - original code
  *          Rémi Denis-Courmont <rem # videolan.org> - EPSV support
@@ -36,10 +36,10 @@
 #include <assert.h>
 
 #include <vlc_access.h>
-#include <vlc_interface.h>
+#include <vlc_dialog.h>
 
 #include <vlc_network.h>
-#include "vlc_url.h"
+#include <vlc_url.h>
 #include <vlc_sout.h>
 
 #ifndef IPPORT_FTP
@@ -68,31 +68,33 @@ static void OutClose( vlc_object_t * );
 #define ACCOUNT_LONGTEXT N_("Account that will be " \
     "used for the connection.")
 
-vlc_module_begin();
-    set_shortname( "FTP" );
-    set_description( N_("FTP input") );
-    set_capability( "access", 0 );
-    set_category( CAT_INPUT );
-    set_subcategory( SUBCAT_INPUT_ACCESS );
+vlc_module_begin ()
+    set_shortname( "FTP" )
+    set_description( N_("FTP input") )
+    set_capability( "access", 0 )
+    set_category( CAT_INPUT )
+    set_subcategory( SUBCAT_INPUT_ACCESS )
     add_integer( "ftp-caching", 2 * DEFAULT_PTS_DELAY / 1000, NULL,
-                 CACHING_TEXT, CACHING_LONGTEXT, true );
+                 CACHING_TEXT, CACHING_LONGTEXT, true )
+        change_safe()
     add_string( "ftp-user", "anonymous", NULL, USER_TEXT, USER_LONGTEXT,
-                false );
+                false )
     add_string( "ftp-pwd", "anonymous@example.com", NULL, PASS_TEXT,
-                PASS_LONGTEXT, false );
+                PASS_LONGTEXT, false )
     add_string( "ftp-account", "anonymous", NULL, ACCOUNT_TEXT,
-                ACCOUNT_LONGTEXT, false );
-    add_shortcut( "ftp" );
-    set_callbacks( InOpen, InClose );
+                ACCOUNT_LONGTEXT, false )
+    add_shortcut( "ftp" )
+    set_callbacks( InOpen, InClose )
 
-    add_submodule();
-    set_shortname( "FTP" );
-    set_description( N_("FTP upload output") );
-    set_capability( "sout access", 0 );
-    set_category( CAT_SOUT );
-    set_subcategory( SUBCAT_SOUT_ACO );
-    set_callbacks( OutOpen, OutClose );
-vlc_module_end();
+    add_submodule ()
+        set_shortname( "FTP" )
+        set_description( N_("FTP upload output") )
+        set_capability( "sout access", 0 )
+        set_category( CAT_SOUT )
+        set_subcategory( SUBCAT_SOUT_ACO )
+        add_shortcut( "ftp" )
+        set_callbacks( OutOpen, OutClose )
+vlc_module_end ()
 
 /*****************************************************************************
  * Local prototypes
@@ -112,6 +114,7 @@ struct access_sys_t
 
     char       sz_epsv_ip[NI_MAXNUMERICHOST];
     bool       out;
+    bool       directory;
 };
 #define GET_OUT_SYS( p_this ) \
     ((access_sys_t *)(((sout_access_out_t *)(p_this))->p_sys))
@@ -132,7 +135,7 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
     if( fd == -1 )
     {
         msg_Err( p_access, "connection failed" );
-        intf_UserFatal( p_access, false, _("Network interaction failed"),
+        dialog_Fatal( p_access, _("Network interaction failed"), "%s",
                         _("VLC could not connect with the given server.") );
         return -1;
     }
@@ -142,7 +145,7 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
     if( i_answer / 100 != 2 )
     {
         msg_Err( p_access, "connection rejected" );
-        intf_UserFatal( p_access, false, _("Network interaction failed"),
+        dialog_Fatal( p_access, _("Network interaction failed"), "%s",
                         _("VLC's connection to the given server was rejected.") );
         return -1;
     }
@@ -206,9 +209,9 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
                     if( i_answer / 100 != 2 )
                     {
                         msg_Err( p_access, "account rejected" );
-                        intf_UserFatal( p_access, false,
-                                        _("Network interaction failed"),
-                                        _("Your account was rejected.") );
+                        dialog_Fatal( p_access,
+                                      _("Network interaction failed"),
+                                      "%s", _("Your account was rejected.") );
                         return -1;
                     }
                     msg_Dbg( p_access, "account accepted" );
@@ -216,16 +219,14 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
 
                 default:
                     msg_Err( p_access, "password rejected" );
-                    intf_UserFatal( p_access, false,
-                                    _("Network interaction failed"),
-                                    _("Your password was rejected.") );
+                    dialog_Fatal( p_access, _("Network interaction failed"),
+                                  "%s",  _("Your password was rejected.") );
                     return -1;
             }
             break;
         default:
             msg_Err( p_access, "user rejected" );
-            intf_UserFatal( p_access, false,
-                        _("Network interaction failed"),
+            dialog_Fatal( p_access, _("Network interaction failed"), "%s",
                         _("Your connection attempt to the server was rejected.") );
             return -1;
     }
@@ -324,6 +325,7 @@ static int InOpen( vlc_object_t *p_this )
     STANDARD_READ_ACCESS_INIT
     p_sys->fd_data = -1;
     p_sys->out = false;
+    p_sys->directory = false;
 
     if( parseURL( &p_sys->url, p_access->psz_path ) )
         goto exit_error;
@@ -332,16 +334,28 @@ static int InOpen( vlc_object_t *p_this )
         goto exit_error;
 
     /* get size */
-    if( ftp_SendCommand( p_this, p_sys, "SIZE %s", p_sys->url.psz_path ? : "" ) < 0 ||
-        ftp_ReadCommand( p_this, p_sys, NULL, &psz_arg ) != 2 )
+    if( ftp_SendCommand( p_this, p_sys, "SIZE %s", p_sys->url.psz_path
+                                               ? p_sys->url.psz_path : "" ) < 0
+     || ftp_ReadCommand( p_this, p_sys, NULL, &psz_arg ) != 2 )
     {
-        msg_Err( p_access, "cannot get file size" );
-        net_Close( p_sys->fd_cmd );
-        goto exit_error;
+        msg_Dbg( p_access, "cannot get file size" );
+        msg_Dbg( p_access, "will try to get directory contents" );
+        if( ftp_SendCommand( p_this, p_sys, "CWD %s", p_sys->url.psz_path
+                             ? p_sys->url.psz_path : "" ) < 0 ||
+        ftp_ReadCommand( p_this, p_sys, NULL, &psz_arg ) != 2 )
+        {
+            msg_Err( p_access, "file or directory doesn't exist" );
+            net_Close( p_sys->fd_cmd );
+            goto exit_error;
+        }
+        p_sys->directory = true;
+    }
+    else
+    {
+        p_access->info.i_size = atoll( &psz_arg[4] );
+        free( psz_arg );
+        msg_Dbg( p_access, "file size: %"PRId64, p_access->info.i_size );
     }
-    p_access->info.i_size = atoll( &psz_arg[4] );
-    free( psz_arg );
-    msg_Dbg( p_access, "file size: %"PRId64, p_access->info.i_size );
 
     /* Start the 'stream' */
     if( ftp_StartStream( p_this, p_sys, 0 ) < 0 )
@@ -367,10 +381,9 @@ static int OutOpen( vlc_object_t *p_this )
     sout_access_out_t *p_access = (sout_access_out_t *)p_this;
     access_sys_t      *p_sys;
 
-    p_sys = malloc( sizeof( *p_sys ) );
-    if( p_sys == NULL )
+    p_sys = calloc( 1, sizeof( *p_sys ) );
+    if( !p_sys )
         return VLC_ENOMEM;
-    memset( p_sys, 0, sizeof( *p_sys ) );
 
     /* Init p_access */
     p_sys->fd_data = -1;
@@ -476,7 +489,6 @@ static int OutSeek( sout_access_out_t *p_access, off_t i_pos )
 static ssize_t Read( access_t *p_access, uint8_t *p_buffer, size_t i_len )
 {
     access_sys_t *p_sys = p_access->p_sys;
-    int i_read;
 
     assert( p_sys->fd_data != -1 );
     assert( !p_sys->out );
@@ -484,14 +496,34 @@ static ssize_t Read( access_t *p_access, uint8_t *p_buffer, size_t i_len )
     if( p_access->info.b_eof )
         return 0;
 
-    i_read = net_Read( p_access, p_sys->fd_data, NULL, p_buffer, i_len,
-                       false );
-    if( i_read == 0 )
-        p_access->info.b_eof = true;
-    else if( i_read > 0 )
-        p_access->info.i_pos += i_read;
+    if( p_sys->directory )
+    {
+        char *psz_line = net_Gets( p_access, p_sys->fd_data, NULL );
+        if( !psz_line )
+        {
+            p_access->info.b_eof = true;
+            return 0;
+        }
+        else
+        {
+            snprintf( (char*)p_buffer, i_len, "ftp://%s:%d/%s/%s\n",
+                      p_sys->url.psz_host, p_sys->url.i_port,
+                      p_sys->url.psz_path, psz_line );
+            free( psz_line );
+            return strlen( (const char *)p_buffer );
+        }
+    }
+    else
+    {
+        int i_read = net_Read( p_access, p_sys->fd_data, NULL,
+                               p_buffer, i_len, false );
+        if( i_read == 0 )
+            p_access->info.b_eof = true;
+        else if( i_read > 0 )
+            p_access->info.i_pos += i_read;
 
-    return i_read;
+        return i_read;
+    }
 }
 
 /*****************************************************************************
@@ -523,10 +555,8 @@ static ssize_t Write( sout_access_out_t *p_access, block_t *p_buffer )
  *****************************************************************************/
 static int Control( access_t *p_access, int i_query, va_list args )
 {
-    bool   *pb_bool;
-    int          *pi_int;
-    int64_t      *pi_64;
-    vlc_value_t  val;
+    bool    *pb_bool;
+    int64_t *pi_64;
 
     switch( i_query )
     {
@@ -549,14 +579,8 @@ static int Control( access_t *p_access, int i_query, va_list args )
             break;
 
         /* */
-        case ACCESS_GET_MTU:
-            pi_int = (int*)va_arg( args, int * );
-            *pi_int = 0;
-            break;
-
         case ACCESS_GET_PTS_DELAY:
             pi_64 = (int64_t*)va_arg( args, int64_t * );
-            var_Get( p_access, "ftp-caching", &val );
             *pi_64 = (int64_t)var_GetInteger( p_access, "ftp-caching" ) * INT64_C(1000);
             break;
 
@@ -763,14 +787,26 @@ static int ftp_StartStream( vlc_object_t *p_access, access_sys_t *p_sys,
     msg_Dbg( p_access, "connection with \"%s:%d\" successful",
              psz_ip, i_port );
 
-    /* "1xx" message */
-    if( ftp_SendCommand( p_access, p_sys, "%s %s",
-                         p_sys->out ? "STOR" : "RETR",
-                         p_sys->url.psz_path ?: "" ) < 0 ||
-        ftp_ReadCommand( p_access, p_sys, &i_answer, NULL ) > 2 )
+    if( p_sys->directory )
     {
-        msg_Err( p_access, "cannot retrieve file" );
+        if( ftp_SendCommand( p_access, p_sys, "NLST" ) < 0 ||
+            ftp_ReadCommand( p_access, p_sys, NULL, &psz_arg ) > 2 )
+        {
+            msg_Err( p_access, "cannot list directory contents" );
         return VLC_EGENERIC;
+        }
+    }
+    else
+    {
+        /* "1xx" message */
+        if( ftp_SendCommand( p_access, p_sys, "%s %s",
+                             p_sys->out ? "STOR" : "RETR",
+                           p_sys->url.psz_path ? p_sys->url.psz_path : "" ) < 0
+         || ftp_ReadCommand( p_access, p_sys, &i_answer, NULL ) > 2 )
+        {
+            msg_Err( p_access, "cannot retrieve file" );
+            return VLC_EGENERIC;
+        }
     }
 
     shutdown( p_sys->fd_data, p_sys->out ? SHUT_RD : SHUT_WR );
