@@ -34,8 +34,6 @@
 
 #define MMS_BUFFER_SIZE 100000
 
-typedef struct mmstu_keepalive_thread_t mmstu_keepalive_thread_t;
-
 struct access_sys_t
 {
     int         i_proto;        /* MMS_PROTO_TCP, MMS_PROTO_UDP */
@@ -96,17 +94,17 @@ struct access_sys_t
     /* misc */
     bool  b_seekable;
 
-    mmstu_keepalive_thread_t *p_keepalive_thread;
     vlc_mutex_t lock_netwrite;
+    struct mmstu_keepalive_t *p_keepalive;
 };
 
-struct mmstu_keepalive_thread_t
+typedef struct mmstu_keepalive_t
 {
-    VLC_COMMON_MEMBERS
-
-    access_t *p_access;
-    bool b_paused;
-    bool b_thread_error;
-};
+    access_t    *p_access;
+    vlc_mutex_t  lock;
+    vlc_cond_t   wait;
+    vlc_thread_t handle;
+    bool         b_paused;
+} mmstu_keepalive_t;
 
 #endif
