@@ -2,7 +2,7 @@
  * bonjour.h
  *****************************************************************************
  * Copyright (C) 2005 the VideoLAN team
- * $Id: bonjour.h 12498 2005-09-09 14:31:22Z jlj $
+ * $Id$
  *
  * Authors: Jon Lech Johansen <jon@nanocrew.net>
  *
@@ -18,8 +18,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-void *bonjour_start_service( vlc_object_t *, char*, char *, int, char * );
+void *bonjour_start_service( vlc_object_t *, const char*, const char *, int,
+                             char * );
 void bonjour_stop_service( void * );
