@@ -2,7 +2,7 @@
  * udp.c
  *****************************************************************************
  * Copyright (C) 2001-2007 the VideoLAN team
- * $Id$
+ * $Id: dae64f1e477c6adf1f2a42e85680276a67252dc7 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -76,21 +76,21 @@ static void Close( vlc_object_t * );
                           "helps reducing the scheduling load on " \
                           "heavily-loaded systems." )
 
-vlc_module_begin();
-    set_description( N_("UDP stream output") );
-    set_shortname( "UDP" );
-    set_category( CAT_SOUT );
-    set_subcategory( SUBCAT_SOUT_ACO );
-    add_integer( SOUT_CFG_PREFIX "caching", DEFAULT_PTS_DELAY / 1000, NULL, CACHING_TEXT, CACHING_LONGTEXT, true );
+vlc_module_begin ()
+    set_description( N_("UDP stream output") )
+    set_shortname( "UDP" )
+    set_category( CAT_SOUT )
+    set_subcategory( SUBCAT_SOUT_ACO )
+    add_integer( SOUT_CFG_PREFIX "caching", DEFAULT_PTS_DELAY / 1000, NULL, CACHING_TEXT, CACHING_LONGTEXT, true )
     add_integer( SOUT_CFG_PREFIX "group", 1, NULL, GROUP_TEXT, GROUP_LONGTEXT,
-                                 true );
-    add_obsolete_integer( SOUT_CFG_PREFIX "late" );
-    add_obsolete_bool( SOUT_CFG_PREFIX "raw" );
+                                 true )
+    add_obsolete_integer( SOUT_CFG_PREFIX "late" )
+    add_obsolete_bool( SOUT_CFG_PREFIX "raw" )
 
-    set_capability( "sout access", 100 );
-    add_shortcut( "udp" );
-    set_callbacks( Open, Close );
-vlc_module_end();
+    set_capability( "sout access", 0 )
+    add_shortcut( "udp" )
+    set_callbacks( Open, Close )
+vlc_module_end ()
 
 /*****************************************************************************
  * Exported prototypes
@@ -113,35 +113,23 @@ static const char *const ppsz_core_options[] = {
 
 static ssize_t Write   ( sout_access_out_t *, block_t * );
 static int  Seek    ( sout_access_out_t *, off_t  );
+static int Control( sout_access_out_t *, int, va_list );
 
-static void* ThreadWrite( vlc_object_t * );
+static void* ThreadWrite( void * );
 static block_t *NewUDPPacket( sout_access_out_t *, mtime_t );
 
-typedef struct sout_access_thread_t
-{
-    VLC_COMMON_MEMBERS
-
-    sout_instance_t *p_sout;
-
-    block_fifo_t *p_fifo;
-
-    int         i_handle;
-
-    int64_t     i_caching;
-    int         i_group;
-
-    block_fifo_t *p_empty_blocks;
-} sout_access_thread_t;
-
 struct sout_access_out_sys_t
 {
-    int                 i_mtu;
+    mtime_t       i_caching;
+    int           i_handle;
     bool          b_mtu_warning;
+    size_t        i_mtu;
 
-    block_t             *p_buffer;
-
-    sout_access_thread_t *p_thread;
+    block_fifo_t *p_fifo;
+    block_fifo_t *p_empty_blocks;
+    block_t      *p_buffer;
 
+    vlc_thread_t  thread;
 };
 
 #define DEFAULT_PORT 1234
@@ -172,7 +160,7 @@ static int Open( vlc_object_t *p_this )
         return VLC_ENOMEM;
     }
 
-    if( !( p_sys = calloc ( 1, sizeof( sout_access_out_sys_t ) ) ) )
+    if( !( p_sys = malloc ( sizeof( *p_sys ) ) ) )
         return VLC_ENOMEM;
     p_access->p_sys = p_sys;
 
@@ -187,29 +175,13 @@ static int Open( vlc_object_t *p_this )
     if (psz_parser[0] == '[')
         psz_parser = strchr (psz_parser, ']');
 
-    psz_parser = strchr (psz_parser ?: psz_dst_addr, ':');
+    psz_parser = strchr (psz_parser ? psz_parser : psz_dst_addr, ':');
     if (psz_parser != NULL)
     {
         *psz_parser++ = '\0';
         i_dst_port = atoi (psz_parser);
     }
 
-    p_sys->p_thread =
-        vlc_object_create( p_access, sizeof( sout_access_thread_t ) );
-    if( !p_sys->p_thread )
-    {
-        free (p_sys);
-        free (psz_dst_addr);
-        return VLC_ENOMEM;
-    }
-
-    vlc_object_attach( p_sys->p_thread, p_access );
-    p_sys->p_thread->p_sout = p_access->p_sout;
-    p_sys->p_thread->b_die  = 0;
-    p_sys->p_thread->b_error= 0;
-    p_sys->p_thread->p_fifo = block_FifoNew();
-    p_sys->p_thread->p_empty_blocks = block_FifoNew();
-
     i_handle = net_ConnectDgram( p_this, psz_dst_addr, i_dst_port, -1,
                                  IPPROTO_UDP );
     free (psz_dst_addr);
@@ -217,7 +189,6 @@ static int Open( vlc_object_t *p_this )
     if( i_handle == -1 )
     {
          msg_Err( p_access, "failed to create raw UDP socket" );
-         vlc_object_release (p_sys->p_thread);
          free (p_sys);
          return VLC_EGENERIC;
     }
@@ -240,32 +211,31 @@ static int Open( vlc_object_t *p_this )
             var_SetInteger (p_access, "dst-port", port);
         }
     }
-    p_sys->p_thread->i_handle = i_handle;
     shutdown( i_handle, SHUT_RD );
 
-    p_sys->p_thread->i_caching =
-        (int64_t)1000 * var_GetInteger( p_access, SOUT_CFG_PREFIX "caching");
-    p_sys->p_thread->i_group =
-        var_GetInteger( p_access, SOUT_CFG_PREFIX "group" );
-
+    p_sys->i_caching = UINT64_C(1000)
+                     * var_GetInteger( p_access, SOUT_CFG_PREFIX "caching");
+    p_sys->i_handle = i_handle;
     p_sys->i_mtu = var_CreateGetInteger( p_this, "mtu" );
+    p_sys->b_mtu_warning = false;
+    p_sys->p_fifo = block_FifoNew();
+    p_sys->p_empty_blocks = block_FifoNew();
     p_sys->p_buffer = NULL;
 
-    if( vlc_thread_create( p_sys->p_thread, "sout write thread", ThreadWrite,
-                           VLC_THREAD_PRIORITY_HIGHEST, false ) )
+    if( vlc_clone( &p_sys->thread, ThreadWrite, p_access,
+                           VLC_THREAD_PRIORITY_HIGHEST ) )
     {
-        msg_Err( p_access->p_sout, "cannot spawn sout access thread" );
+        msg_Err( p_access, "cannot spawn sout access thread" );
+        block_FifoRelease( p_sys->p_fifo );
+        block_FifoRelease( p_sys->p_empty_blocks );
         net_Close (i_handle);
-        vlc_object_release( p_sys->p_thread );
         free (p_sys);
         return VLC_EGENERIC;
     }
 
     p_access->pf_write = Write;
     p_access->pf_seek = Seek;
-
-    /* update p_sout->i_out_pace_nocontrol */
-    p_access->p_sout->i_out_pace_nocontrol++;
+    p_access->pf_control = Control;
 
     return VLC_SUCCESS;
 }
@@ -277,38 +247,34 @@ static void Close( vlc_object_t * p_this )
 {
     sout_access_out_t     *p_access = (sout_access_out_t*)p_this;
     sout_access_out_sys_t *p_sys = p_access->p_sys;
-    int i;
 
-    vlc_object_kill( p_sys->p_thread );
-    block_FifoWake( p_sys->p_thread->p_fifo );
-
-    for( i = 0; i < 10; i++ )
-    {
-        block_t *p_dummy = block_New( p_access, p_sys->i_mtu );
-        p_dummy->i_dts = 0;
-        p_dummy->i_pts = 0;
-        p_dummy->i_length = 0;
-        memset( p_dummy->p_buffer, 0, p_dummy->i_buffer );
-        block_FifoPut( p_sys->p_thread->p_fifo, p_dummy );
-    }
-    vlc_thread_join( p_sys->p_thread );
-
-    block_FifoRelease( p_sys->p_thread->p_fifo );
-    block_FifoRelease( p_sys->p_thread->p_empty_blocks );
+    vlc_cancel( p_sys->thread );
+    vlc_join( p_sys->thread, NULL );
+    block_FifoRelease( p_sys->p_fifo );
+    block_FifoRelease( p_sys->p_empty_blocks );
 
     if( p_sys->p_buffer ) block_Release( p_sys->p_buffer );
 
-    net_Close( p_sys->p_thread->i_handle );
-
-    vlc_object_detach( p_sys->p_thread );
-    vlc_object_release( p_sys->p_thread );
-    /* update p_sout->i_out_pace_nocontrol */
-    p_access->p_sout->i_out_pace_nocontrol--;
-
-    msg_Dbg( p_access, "UDP access output closed" );
+    net_Close( p_sys->i_handle );
     free( p_sys );
 }
 
+static int Control( sout_access_out_t *p_access, int i_query, va_list args )
+{
+    (void)p_access;
+
+    switch( i_query )
+    {
+        case ACCESS_OUT_CONTROLS_PACE:
+            *va_arg( args, bool * ) = false;
+            break;
+
+        default:
+            return VLC_EGENERIC;
+    }
+    return VLC_SUCCESS;
+}
+
 /*****************************************************************************
  * Write: standard write on a file descriptor.
  *****************************************************************************/
@@ -334,22 +300,21 @@ static ssize_t Write( sout_access_out_t *p_access, block_t *p_buffer )
         if( p_sys->p_buffer &&
             p_sys->p_buffer->i_buffer + p_buffer->i_buffer > p_sys->i_mtu )
         {
-            if( p_sys->p_buffer->i_dts + p_sys->p_thread->i_caching < now )
+            if( p_sys->p_buffer->i_dts + p_sys->i_caching < now )
             {
                 msg_Dbg( p_access, "late packet for UDP input (%"PRId64 ")",
                          now - p_sys->p_buffer->i_dts
-                          - p_sys->p_thread->i_caching );
+                          - p_sys->i_caching );
             }
-            block_FifoPut( p_sys->p_thread->p_fifo, p_sys->p_buffer );
+            block_FifoPut( p_sys->p_fifo, p_sys->p_buffer );
             p_sys->p_buffer = NULL;
         }
 
         i_len += p_buffer->i_buffer;
         while( p_buffer->i_buffer )
         {
-            int i_payload_size = p_sys->i_mtu;
-
-            int i_write = __MIN( p_buffer->i_buffer, i_payload_size );
+            size_t i_payload_size = p_sys->i_mtu;
+            size_t i_write = __MIN( p_buffer->i_buffer, i_payload_size );
 
             i_packets++;
 
@@ -375,13 +340,13 @@ static ssize_t Write( sout_access_out_t *p_access, block_t *p_buffer )
             if( p_sys->p_buffer->i_buffer == p_sys->i_mtu || i_packets > 1 )
             {
                 /* Flush */
-                if( p_sys->p_buffer->i_dts + p_sys->p_thread->i_caching < now )
+                if( p_sys->p_buffer->i_dts + p_sys->i_caching < now )
                 {
                     msg_Dbg( p_access, "late packet for udp input (%"PRId64 ")",
                              mdate() - p_sys->p_buffer->i_dts
-                              - p_sys->p_thread->i_caching );
+                              - p_sys->i_caching );
                 }
-                block_FifoPut( p_sys->p_thread->p_fifo, p_sys->p_buffer );
+                block_FifoPut( p_sys->p_fifo, p_sys->p_buffer );
                 p_sys->p_buffer = NULL;
             }
         }
@@ -391,7 +356,7 @@ static ssize_t Write( sout_access_out_t *p_access, block_t *p_buffer )
         p_buffer = p_next;
     }
 
-    return( p_sys->p_thread->b_error ? -1 : i_len );
+    return i_len;
 }
 
 /*****************************************************************************
@@ -399,6 +364,7 @@ static ssize_t Write( sout_access_out_t *p_access, block_t *p_buffer )
  *****************************************************************************/
 static int Seek( sout_access_out_t *p_access, off_t i_pos )
 {
+    (void) i_pos;
     msg_Err( p_access, "UDP sout access cannot seek" );
     return -1;
 }
@@ -411,19 +377,19 @@ static block_t *NewUDPPacket( sout_access_out_t *p_access, mtime_t i_dts)
     sout_access_out_sys_t *p_sys = p_access->p_sys;
     block_t *p_buffer;
 
-    while ( block_FifoCount( p_sys->p_thread->p_empty_blocks ) > MAX_EMPTY_BLOCKS )
+    while ( block_FifoCount( p_sys->p_empty_blocks ) > MAX_EMPTY_BLOCKS )
     {
-        p_buffer = block_FifoGet( p_sys->p_thread->p_empty_blocks );
+        p_buffer = block_FifoGet( p_sys->p_empty_blocks );
         block_Release( p_buffer );
     }
 
-    if( block_FifoCount( p_sys->p_thread->p_empty_blocks ) == 0 )
+    if( block_FifoCount( p_sys->p_empty_blocks ) == 0 )
     {
-        p_buffer = block_New( p_access->p_sout, p_sys->i_mtu );
+        p_buffer = block_Alloc( p_sys->i_mtu );
     }
     else
     {
-        p_buffer = block_FifoGet(p_sys->p_thread->p_empty_blocks );
+        p_buffer = block_FifoGet(p_sys->p_empty_blocks );
         p_buffer->i_flags = 0;
         p_buffer = block_Realloc( p_buffer, 0, p_sys->i_mtu );
     }
@@ -437,43 +403,31 @@ static block_t *NewUDPPacket( sout_access_out_t *p_access, mtime_t i_dts)
 /*****************************************************************************
  * ThreadWrite: Write a packet on the network at the good time.
  *****************************************************************************/
-static void* ThreadWrite( vlc_object_t *p_this )
+static void* ThreadWrite( void *data )
 {
-    sout_access_thread_t *p_thread = (sout_access_thread_t*)p_this;
-    mtime_t              i_date_last = -1;
-    mtime_t              i_to_send = p_thread->i_group;
-    int                  i_dropped_packets = 0;
+    sout_access_out_t *p_access = data;
+    sout_access_out_sys_t *p_sys = p_access->p_sys;
+    mtime_t i_date_last = -1;
+    const unsigned i_group = var_GetInteger( p_access,
+                                             SOUT_CFG_PREFIX "group" );
+    mtime_t i_to_send = i_group;
+    unsigned i_dropped_packets = 0;
 
-    while( vlc_object_alive (p_thread) )
+    for (;;)
     {
-        block_t *p_pk;
+        block_t *p_pk = block_FifoGet( p_sys->p_fifo );
         mtime_t       i_date, i_sent;
-#if 0
-        if( (i++ % 1000)==0 ) {
-          int i = 0;
-          int j = 0;
-          block_t *p_tmp = p_thread->p_empty_blocks->p_first;
-          while( p_tmp ) { p_tmp = p_tmp->p_next; i++;}
-          p_tmp = p_thread->p_fifo->p_first;
-          while( p_tmp ) { p_tmp = p_tmp->p_next; j++;}
-          msg_Dbg( p_thread, "fifo depth: %d/%d, empty blocks: %d/%d",
-                   p_thread->p_fifo->i_depth, j,p_thread->p_empty_blocks->i_depth,i );
-        }
-#endif
-        p_pk = block_FifoGet( p_thread->p_fifo );
-        if( p_pk == NULL )
-            continue; /* forced wake-up */
 
-        i_date = p_thread->i_caching + p_pk->i_dts;
+        i_date = p_sys->i_caching + p_pk->i_dts;
         if( i_date_last > 0 )
         {
             if( i_date - i_date_last > 2000000 )
             {
                 if( !i_dropped_packets )
-                    msg_Dbg( p_thread, "mmh, hole (%"PRId64" > 2s) -> drop",
+                    msg_Dbg( p_access, "mmh, hole (%"PRId64" > 2s) -> drop",
                              i_date - i_date_last );
 
-                block_FifoPut( p_thread->p_empty_blocks, p_pk );
+                block_FifoPut( p_sys->p_empty_blocks, p_pk );
 
                 i_date_last = i_date;
                 i_dropped_packets++;
@@ -482,27 +436,25 @@ static void* ThreadWrite( vlc_object_t *p_this )
             else if( i_date - i_date_last < -1000 )
             {
                 if( !i_dropped_packets )
-                    msg_Dbg( p_thread, "mmh, packets in the past (%"PRId64")",
+                    msg_Dbg( p_access, "mmh, packets in the past (%"PRId64")",
                              i_date_last - i_date );
             }
         }
 
+        block_cleanup_push( p_pk );
         i_to_send--;
         if( !i_to_send || (p_pk->i_flags & BLOCK_FLAG_CLOCK) )
         {
             mwait( i_date );
-            i_to_send = p_thread->i_group;
-        }
-        ssize_t val = send( p_thread->i_handle, p_pk->p_buffer,
-                            p_pk->i_buffer, 0 );
-        if (val == -1)
-        {
-            msg_Warn( p_thread, "send error: %m" );
+            i_to_send = i_group;
         }
+        if ( send( p_sys->i_handle, p_pk->p_buffer, p_pk->i_buffer, 0 ) == -1 )
+            msg_Warn( p_access, "send error: %m" );
+        vlc_cleanup_pop();
 
         if( i_dropped_packets )
         {
-            msg_Dbg( p_thread, "dropped %i packets", i_dropped_packets );
+            msg_Dbg( p_access, "dropped %i packets", i_dropped_packets );
             i_dropped_packets = 0;
         }
 
@@ -510,12 +462,12 @@ static void* ThreadWrite( vlc_object_t *p_this )
         i_sent = mdate();
         if ( i_sent > i_date + 20000 )
         {
-            msg_Dbg( p_thread, "packet has been sent too late (%"PRId64 ")",
+            msg_Dbg( p_access, "packet has been sent too late (%"PRId64 ")",
                      i_sent - i_date );
         }
 #endif
 
-        block_FifoPut( p_thread->p_empty_blocks, p_pk );
+        block_FifoPut( p_sys->p_empty_blocks, p_pk );
 
         i_date_last = i_date;
     }
