@@ -1,8 +1,8 @@
 /*****************************************************************************
- * simple.c : simple channel mixer plug-in (only 7/7.1/5/5.1 -> Stereo for now)
+ * simple.c : simple channel mixer plug-in
  *****************************************************************************
  * Copyright (C) 2002, 2006 the VideoLAN team
- * $Id: 7fa42580da2a0bd97401a69bebed518b54f8491b $
+ * $Id: 0f0cb6b7ec4658e603c31ae494311f7af3fbcf17 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -33,6 +33,7 @@
 #include <vlc_aout.h>
 #include <vlc_filter.h>
 #include <vlc_block.h>
+#include <assert.h>
 
 /*****************************************************************************
  * Module descriptor
@@ -40,18 +41,18 @@
 static int  Create    ( vlc_object_t * );
 static int  OpenFilter( vlc_object_t * );
 
-vlc_module_begin();
-    set_description( N_("Audio filter for simple channel mixing") );
-    set_capability( "audio filter", 10 );
-    set_category( CAT_AUDIO );
-    set_subcategory( SUBCAT_AUDIO_MISC );
-    set_callbacks( Create, NULL );
+vlc_module_begin ()
+    set_description( N_("Audio filter for simple channel mixing") )
+    set_capability( "audio filter", 10 )
+    set_category( CAT_AUDIO )
+    set_subcategory( SUBCAT_AUDIO_MISC )
+    set_callbacks( Create, NULL )
 
-    add_submodule();
-    set_description( N_("audio filter for simple channel mixing") );
-    set_capability( "audio filter2", 10 );
-    set_callbacks( OpenFilter, NULL );
-vlc_module_end();
+    add_submodule ()
+    set_description( N_("audio filter for simple channel mixing") )
+    set_capability( "audio filter2", 10 )
+    set_callbacks( OpenFilter, NULL )
+vlc_module_end ()
 
 /*****************************************************************************
  * Local prototypes
@@ -60,11 +61,14 @@ vlc_module_end();
 #define AOUT_CHANS_STEREO_REAR   ( AOUT_CHAN_REARLEFT | AOUT_CHAN_REARRIGHT )
 #define AOUT_CHANS_STEREO_MIDDLE (AOUT_CHAN_MIDDLELEFT | AOUT_CHAN_MIDDLERIGHT )
 
-#define AOUT_CHANS_2_0 AOUT_CHANS_STEREO_FRONT
-#define AOUT_CHANS_4_0 (AOUT_CHANS_STEREO_FRONT | AOUT_CHANS_STEREO_REAR )
-#define AOUT_CHANS_5_0 ( AOUT_CHANS_4_0 | AOUT_CHAN_CENTER )
-#define AOUT_CHANS_6_0 (AOUT_CHANS_STEREO_FRONT | AOUT_CHANS_STEREO_REAR | AOUT_CHANS_STEREO_MIDDLE )
-#define AOUT_CHANS_7_0 ( AOUT_CHANS_6_0 | AOUT_CHAN_CENTER )
+#define AOUT_CHANS_2_0          AOUT_CHANS_STEREO_FRONT
+#define AOUT_CHANS_3_0          ( AOUT_CHANS_STEREO_FRONT | AOUT_CHAN_CENTER )
+#define AOUT_CHANS_4_0          ( AOUT_CHANS_STEREO_FRONT | AOUT_CHANS_STEREO_REAR )
+#define AOUT_CHANS_4_0_MIDDLE   ( AOUT_CHANS_STEREO_FRONT | AOUT_CHANS_STEREO_MIDDLE )
+#define AOUT_CHANS_5_0          ( AOUT_CHANS_4_0 | AOUT_CHAN_CENTER )
+#define AOUT_CHANS_5_0_MIDDLE   ( AOUT_CHANS_4_0_MIDDLE | AOUT_CHAN_CENTER )
+#define AOUT_CHANS_6_0          ( AOUT_CHANS_STEREO_FRONT | AOUT_CHANS_STEREO_REAR | AOUT_CHANS_STEREO_MIDDLE )
+#define AOUT_CHANS_7_0          ( AOUT_CHANS_6_0 | AOUT_CHAN_CENTER )
 
 static bool IsSupported( const audio_format_t *p_input, const audio_format_t *p_output );
 
@@ -96,6 +100,14 @@ static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
                     aout_buffer_t * p_in_buf, aout_buffer_t * p_out_buf )
 {
     VLC_UNUSED(p_aout);
+    const unsigned i_input_physical = p_filter->input.i_physical_channels;
+
+    const bool b_input_7_0 = (i_input_physical & ~AOUT_CHAN_LFE) == AOUT_CHANS_7_0;
+    const bool b_input_5_0 = !b_input_7_0 &&
+                             ( (i_input_physical & AOUT_CHANS_5_0) == AOUT_CHANS_5_0 ||
+                               (i_input_physical & AOUT_CHANS_5_0_MIDDLE) == AOUT_CHANS_5_0_MIDDLE );
+    const bool b_input_3_0 = !b_input_7_0 && !b_input_5_0 &&
+                             (i_input_physical & ~AOUT_CHAN_LFE) == AOUT_CHANS_3_0;
     int i_input_nb = aout_FormatNbChannels( &p_filter->input );
     int i_output_nb = aout_FormatNbChannels( &p_filter->output );
     float *p_dest = (float *)p_out_buf->p_buffer;
@@ -107,7 +119,7 @@ static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
 
     if( p_filter->output.i_physical_channels == AOUT_CHANS_2_0 )
     {
-        if( p_filter->input.i_physical_channels & AOUT_CHAN_MIDDLELEFT )
+        if( b_input_7_0 )
         for( i = p_in_buf->i_nb_samples; i--; )
         {
             *p_dest = p_src[6] + 0.5 * p_src[0] + p_src[2] / 4 + p_src[4] / 4;
@@ -119,7 +131,7 @@ static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
 
             if( p_filter->input.i_physical_channels & AOUT_CHAN_LFE ) p_src++;
         }
-        else
+        else if( b_input_5_0 )
         for( i = p_in_buf->i_nb_samples; i--; )
         {
             *p_dest = p_src[4] + 0.5 * p_src[0] + 0.33 * p_src[2];
@@ -129,12 +141,24 @@ static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
 
             p_src += 5;
 
+            if( p_filter->input.i_physical_channels & AOUT_CHAN_LFE ) p_src++;
+        }
+        else if( b_input_3_0 )
+        for( i = p_in_buf->i_nb_samples; i--; )
+        {
+            *p_dest = p_src[2] + 0.5 * p_src[0];
+            p_dest++;
+            *p_dest = p_src[2] + 0.5 * p_src[1];
+            p_dest++;
+
+            p_src += 3;
+
             if( p_filter->input.i_physical_channels & AOUT_CHAN_LFE ) p_src++;
         }
     }
     else if( p_filter->output.i_physical_channels == AOUT_CHAN_CENTER )
     {
-        if( p_filter->input.i_physical_channels & AOUT_CHAN_MIDDLELEFT )
+        if( b_input_7_0 )
         for( i = p_in_buf->i_nb_samples; i--; )
         {
             *p_dest = p_src[6] + p_src[0] / 4 + p_src[1] / 4 + p_src[2] / 8 + p_src[3] / 8 + p_src[4] / 8 + p_src[5] / 8;
@@ -144,7 +168,7 @@ static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
 
             if( p_filter->input.i_physical_channels & AOUT_CHAN_LFE ) p_src++;
         }
-        else if( p_filter->input.i_physical_channels & AOUT_CHAN_REARLEFT )
+        else if( b_input_5_0 )
         for( i = p_in_buf->i_nb_samples; i--; )
         {
             *p_dest = p_src[4] + p_src[0] / 4 + p_src[1] / 4 + p_src[2] / 6 + p_src[3] / 6;
@@ -154,6 +178,16 @@ static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
 
             if( p_filter->input.i_physical_channels & AOUT_CHAN_LFE ) p_src++;
         }
+        else if( b_input_3_0 )
+        for( i = p_in_buf->i_nb_samples; i--; )
+        {
+            *p_dest = p_src[2] + p_src[0] / 4 + p_src[1] / 4;
+            p_dest++;
+
+            p_src += 3;
+
+            if( p_filter->input.i_physical_channels & AOUT_CHAN_LFE ) p_src++;
+        }
         else
         for( i = p_in_buf->i_nb_samples; i--; )
         {
@@ -165,7 +199,10 @@ static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
     }
     else
     {
-        if( p_filter->input.i_physical_channels & AOUT_CHAN_MIDDLELEFT )
+        assert( p_filter->output.i_physical_channels == AOUT_CHANS_4_0 );
+        assert( b_input_7_0 || b_input_5_0 );
+
+        if( b_input_7_0 )
         for( i = p_in_buf->i_nb_samples; i--; )
         {
             *p_dest = p_src[6] + 0.5 * p_src[0] + p_src[2] / 6;
@@ -302,9 +339,12 @@ static bool IsSupported( const audio_format_t *p_input, const audio_format_t *p_
         return false;
     }
 
-    /* Only from 7/7.1/5/5.1/2.0 */
+    /* Only from 7/7.1/5/5.1/3/3.1/2.0
+     * XXX 5.X rear and middle are handled the same way */
     if( (p_input->i_physical_channels & ~AOUT_CHAN_LFE) != AOUT_CHANS_7_0 &&
         (p_input->i_physical_channels & ~AOUT_CHAN_LFE) != AOUT_CHANS_5_0 &&
+        (p_input->i_physical_channels & ~AOUT_CHAN_LFE) != AOUT_CHANS_5_0_MIDDLE &&
+        (p_input->i_physical_channels & ~AOUT_CHAN_LFE) != AOUT_CHANS_3_0 &&
          p_input->i_physical_channels != AOUT_CHANS_2_0 )
     {
         return false;
