@@ -40,20 +40,18 @@
 #endif
 
 #include <vlc_codec.h>
-#include <vlc_input.h>
 #include <vlc_interface.h>
-#include <vlc_vout.h>
 #include <vlc_playlist.h>
 #include <vlc_osd.h>
 
-#include "vlc_keys.h"
+#include <vlc_keys.h>
 
 #include "browser_open.h"
 #include "history.h"
 #include "xstrcat.h"
 #include "xurl.h"
 
-#undef  CMML_INTF_USE_TIMED_URIS
+#undef CMML_INTF_USE_TIMED_URIS
 
 #undef  CMML_INTF_DEBUG
 #undef  CMML_INTF_HISTORY_DEBUG
@@ -63,6 +61,7 @@
  *****************************************************************************/
 struct intf_sys_t
 {
+    vlc_mutex_t         lock;
     decoder_t *         p_cmml_decoder;
     input_thread_t *    p_input;
 
@@ -100,8 +99,10 @@ static int   GoForwardCallback          ( vlc_object_t *, char const *,
                                           vlc_value_t, vlc_value_t, void * );
 
 static char *GetTimedURLFromPlaylistItem( intf_thread_t *, playlist_item_t * );
-static char *GetTimedURIFragmentForTime ( int );
+#ifdef CMML_INTF_USE_TIMED_URIS
 static int   GetCurrentTimeInSeconds    ( input_thread_t * );
+static char *GetTimedURIFragmentForTime ( int );
+#endif
 static int   DisplayAnchor              ( intf_thread_t *, vout_thread_t *,
                                           char *, char * );
 static int   DisplayPendingAnchor       ( intf_thread_t *, vout_thread_t * );
@@ -120,11 +121,10 @@ int OpenIntf ( vlc_object_t *p_this )
 
     p_intf->p_sys = malloc( sizeof( intf_sys_t ) );
     if( p_intf->p_sys == NULL )
-    {
-        return( 1 );
-    };
+        return VLC_ENOMEM;
 
     p_intf->pf_run = RunIntf;
+    vlc_mutex_init( &p_intf->p_sys->lock );
 
     var_AddCallback( p_intf->p_libvlc, "key-action", KeyEvent, p_intf );
     /* we also need to add the callback for "mouse-clicked", but do that later
@@ -140,7 +140,7 @@ int OpenIntf ( vlc_object_t *p_this )
     var_AddCallback( p_intf->p_libvlc, "browse-follow-anchor",
                      FollowAnchorCallback, p_intf );
 
-    return( 0 );
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
@@ -168,6 +168,7 @@ void CloseIntf ( vlc_object_t *p_this )
 
     vlc_object_release( p_intf->p_sys->p_cmml_decoder );
 
+    vlc_mutex_destroy( &p_intf->p_sys->lock );
     free( p_intf->p_sys );
 }
 
@@ -177,6 +178,7 @@ void CloseIntf ( vlc_object_t *p_this )
  *****************************************************************************/
 static void RunIntf( intf_thread_t *p_intf )
 {
+    int canc = vlc_savecancel();
     vout_thread_t * p_vout = NULL;
 
     if( InitThread( p_intf ) < 0 )
@@ -213,7 +215,7 @@ static void RunIntf( intf_thread_t *p_intf )
             }
         }
 
-        vlc_mutex_lock( &p_intf->change_lock );
+        vlc_mutex_lock( &p_intf->p_sys->lock );
 
         /*
          * keyboard event
@@ -233,7 +235,7 @@ static void RunIntf( intf_thread_t *p_intf )
                 break;
         }
         p_intf->p_sys->i_key_action = 0;
-        vlc_mutex_unlock( &p_intf->change_lock );
+        vlc_mutex_unlock( &p_intf->p_sys->lock );
 
         (void) DisplayPendingAnchor( p_intf, p_vout );
 
@@ -250,6 +252,7 @@ static void RunIntf( intf_thread_t *p_intf )
     }
 
     vlc_object_release( p_intf->p_sys->p_input );
+    vlc_restorecancel( canc );
 }
 
 /*****************************************************************************
@@ -285,25 +288,6 @@ static int DisplayPendingAnchor( intf_thread_t *p_intf, vout_thread_t *p_vout )
 
     if( p_vout != NULL )
     {
-        /* don't display anchor if main interface can display it */
-        p_primary_intf = vlc_object_find( p_intf->p_libvlc, VLC_OBJECT_INTF,
-                FIND_CHILD );
-
-        if( p_primary_intf )
-        {
-            if( var_Get( p_primary_intf, "intf-displays-cmml-description", &val )
-                    == VLC_SUCCESS )
-            {
-                if( val.b_bool == true )
-                {
-                    vlc_object_release( p_primary_intf );
-                    return true;
-                }
-            }
-
-            vlc_object_release( p_primary_intf );
-        }
-
         /* display anchor as subtitle on-screen */
         if( DisplayAnchor( p_intf, p_vout, psz_description, psz_url )
                 != VLC_SUCCESS )
@@ -353,14 +337,14 @@ static int InitThread( intf_thread_t * p_intf )
             return VLC_EGENERIC;
         }
 
-        vlc_mutex_lock( &p_intf->change_lock );
+        vlc_mutex_lock( &p_intf->p_sys->lock );
 
         p_intf->p_sys->p_input = p_input;
         p_intf->p_sys->p_cmml_decoder = p_cmml_decoder;
 
         p_intf->p_sys->i_key_action = 0;
 
-        vlc_mutex_unlock( &p_intf->change_lock );
+        vlc_mutex_unlock( &p_intf->p_sys->lock );
 
         return VLC_SUCCESS;
     }
@@ -393,11 +377,13 @@ static int KeyEvent( vlc_object_t *p_this, char const *psz_var,
     VLC_UNUSED(p_this); VLC_UNUSED(psz_var);
     VLC_UNUSED(oldval); VLC_UNUSED(newval);
     intf_thread_t *p_intf = (intf_thread_t *)p_data;
-    vlc_mutex_lock( &p_intf->change_lock );
 
+
+    vlc_mutex_lock( &p_intf->p_sys->lock );
+    /* FIXME: key presses might get lost here... */
     p_intf->p_sys->i_key_action = newval.i_int;
 
-    vlc_mutex_unlock( &p_intf->change_lock );
+    vlc_mutex_unlock( &p_intf->p_sys->lock );
 
     return VLC_SUCCESS;
 }
@@ -435,10 +421,10 @@ static void FollowAnchor ( intf_thread_t *p_intf )
         mtime_t i_seconds;
         vlc_value_t time;
 
-        p_playlist = pl_Yield( p_intf );
+        p_playlist = pl_Hold( p_intf );
 
         /* Get new URL */
-        p_current_item = p_playlist->status.p_item;
+        p_current_item = playlist_CurrentPlayingItem( p_playlist );
         char *psz_uri = input_item_GetURI( p_current_item->p_input );
 #ifdef CMML_INTF_DEBUG
         msg_Dbg( p_intf, "Current playlist item URL is \"%s\"", psz_uri );
@@ -515,7 +501,7 @@ static void FollowAnchor ( intf_thread_t *p_intf )
 
         free( psz_uri_to_load );
 
-        vlc_object_release( p_playlist );
+        pl_Release( p_intf );
     }
 }
 
@@ -552,17 +538,14 @@ char *GetTimedURLFromPlaylistItem( intf_thread_t *p_intf,
     return psz_return_value;
 #else
     VLC_UNUSED(p_intf);
-    void *p;
-
-    /* Suppress warning messages about unused functions */
-    p = GetTimedURIFragmentForTime; /* unused */
-    p = GetCurrentTimeInSeconds;    /* unused */
 
     return input_item_GetURI( p_current_item->p_input );
 #endif
 }
 
 
+
+#ifdef CMML_INTF_USE_TIMED_URIS
 /*
  * Get the current time, rounded down to the nearest second
  *
@@ -588,6 +571,7 @@ char *GetTimedURIFragmentForTime( int seconds )
         return NULL;
     return psz_time;
 }
+#endif
 
 static
 int GoBackCallback( vlc_object_t *p_this, char const *psz_var,
@@ -639,7 +623,7 @@ void GoBack( intf_thread_t *p_intf )
 #endif
 
     /* Find the playlist */
-    p_playlist = pl_Yield( p_intf );
+    p_playlist = pl_Hold( p_intf );
 
     /* Retrieve navigation history from playlist */
     if( var_Get( p_playlist, "navigation-history", &history ) != VLC_SUCCESS ||
@@ -647,7 +631,7 @@ void GoBack( intf_thread_t *p_intf )
     {
         /* History doesn't exist yet: ignore user's request */
         msg_Warn( p_intf, "can't go back: no history exists yet" );
-        vlc_object_release( p_playlist );
+        pl_Release( p_intf );
         return;
     }
 
@@ -662,11 +646,11 @@ void GoBack( intf_thread_t *p_intf )
     if( history_CanGoBack( p_history ) == false )
     {
         msg_Warn( p_intf, "can't go back: already at beginning of history" );
-        vlc_object_release( p_playlist );
+        pl_Release( p_intf );
         return;
     }
 
-    p_current_item = p_playlist->status.p_item;
+    p_current_item = playlist_CurrentPlayingItem( p_playlist );
 
     /* Save the currently-playing media in a new history item */
     psz_timed_url = GetTimedURLFromPlaylistItem( p_intf, p_current_item );
@@ -678,7 +662,7 @@ void GoBack( intf_thread_t *p_intf )
 #ifdef CMML_INTF_DEBUG
         msg_Dbg( p_intf, "back: could not initialise new history item" );
 #endif
-        vlc_object_release( p_playlist );
+        pl_Release( p_intf );
         return;
     }
 
@@ -693,7 +677,7 @@ void GoBack( intf_thread_t *p_intf )
 #endif
 
     ReplacePlaylistItem( p_playlist, p_history_item->psz_uri );
-    vlc_object_release( p_playlist );
+    pl_Release( p_intf );
 }
 
 static
@@ -711,7 +695,7 @@ void GoForward( intf_thread_t *p_intf )
 #endif
 
     /* Find the playlist */
-    p_playlist = pl_Yield( p_intf );
+    p_playlist = pl_Hold( p_intf );
 
     /* Retrieve navigation history from playlist */
     if( var_Get( p_playlist, "navigation-history", &history ) != VLC_SUCCESS ||
@@ -719,7 +703,7 @@ void GoForward( intf_thread_t *p_intf )
     {
         /* History doesn't exist yet: ignore user's request */
         msg_Warn( p_intf, "can't go back: no history exists yet" );
-        vlc_object_release( p_playlist );
+        pl_Release( p_intf );
         return;
     }
 
@@ -734,7 +718,7 @@ void GoForward( intf_thread_t *p_intf )
     if( history_CanGoForward( p_history ) == false )
     {
         msg_Warn( p_intf, "can't go forward: already at end of history" );
-        vlc_object_release( p_playlist );
+        pl_Release( p_intf );
         return;
     }
 
@@ -745,10 +729,10 @@ void GoForward( intf_thread_t *p_intf )
 #ifdef CMML_INTF_DEBUG
         msg_Dbg( p_intf, "forward: could not initialise new history item" );
 #endif
-        vlc_object_release( p_playlist );
+        pl_Release( p_intf );
         return;
     }
-    p_current_item = p_playlist->status.p_item;
+    p_current_item = playlist_CurrentPlayingItem( p_playlist );
     p_new_history_item->psz_uri = GetTimedURLFromPlaylistItem( p_intf,
             p_current_item );
     p_new_history_item->psz_name = p_new_history_item->psz_uri;
@@ -764,7 +748,7 @@ void GoForward( intf_thread_t *p_intf )
 #endif
 
     ReplacePlaylistItem( p_playlist, p_history_item->psz_uri );
-    vlc_object_release( p_playlist );
+    pl_Release( p_intf );
 }
 
 static void ReplacePlaylistItem( playlist_t *p_playlist, char *psz_uri )
