@@ -2,7 +2,7 @@
  * fluidsynth.c: Software MIDI synthetizer using libfluidsynth
  *****************************************************************************
  * Copyright © 2007 Rémi Denis-Courmont
- * $Id: 600353fa3ca9604f34bade2c72691a79ce6fecc3 $
+ * $Id: 26bc6e95888390bce660d68fa88c5b27bc79b3e6 $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,15 +37,16 @@
 static int  Open  (vlc_object_t *);
 static void Close (vlc_object_t *);
 
-vlc_module_begin();
-    set_description (N_("FluidSynth MIDI synthetizer"));
-    set_capability ("decoder", 100);
-    set_category (CAT_INPUT);
-    set_subcategory (SUBCAT_INPUT_ACODEC);
-    set_callbacks (Open, Close);
+vlc_module_begin ()
+    set_description (N_("FluidSynth MIDI synthetizer"))
+    set_capability ("decoder", 100)
+    set_shortname (N_("FluidSynth"))
+    set_category (CAT_INPUT)
+    set_subcategory (SUBCAT_INPUT_ACODEC)
+    set_callbacks (Open, Close)
     add_file ("soundfont", "", NULL,
               SOUNDFONT_TEXT, SOUNDFONT_LONGTEXT, false);
-vlc_module_end();
+vlc_module_end ()
 
 
 struct decoder_sys_t
@@ -127,12 +128,14 @@ static aout_buffer_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block)
 {
     block_t *p_block;
     decoder_sys_t *p_sys = p_dec->p_sys;
+    aout_buffer_t *p_out = NULL;
 
     if (pp_block == NULL)
         return NULL;
     p_block = *pp_block;
     if (p_block == NULL)
         return NULL;
+    *pp_block = NULL;
 
     if (p_block->i_pts && !aout_DateGet (&p_sys->end_date))
         aout_DateSet (&p_sys->end_date, p_block->i_pts);
@@ -140,12 +143,11 @@ static aout_buffer_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block)
     if (p_block->i_pts < aout_DateGet (&p_sys->end_date))
     {
         msg_Warn (p_dec, "MIDI message in the past?");
-        block_Release (p_block);
-        return NULL;
+        goto drop;
     }
 
     if (p_block->i_buffer < 1)
-        return NULL;
+        goto drop;
 
     uint8_t channel = p_block->p_buffer[0] & 0xf;
     uint8_t p1 = (p_block->i_buffer > 1) ? (p_block->p_buffer[1] & 0x7f) : 0;
@@ -169,25 +171,22 @@ static aout_buffer_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block)
             fluid_synth_pitch_bend (p_sys->synth, channel, (p1 << 7) | p2);
             break;
     }
-    p_block->p_buffer += p_block->i_buffer;
-    p_block->i_buffer = 0;
 
     unsigned samples =
         (p_block->i_pts - aout_DateGet (&p_sys->end_date)) * 441 / 10000;
     if (samples == 0)
         return NULL;
 
-    aout_buffer_t *p_out = p_dec->pf_aout_buffer_new (p_dec, samples);
+    p_out = decoder_NewAudioBuffer (p_dec, samples);
     if (p_out == NULL)
-    {
-        block_Release (p_block);
-        return NULL;
-    }
+        goto drop;
 
     p_out->start_date = aout_DateGet (&p_sys->end_date );
     p_out->end_date   = aout_DateIncrement (&p_sys->end_date, samples);
     fluid_synth_write_float (p_sys->synth, samples,
                              p_out->p_buffer, 0, 2,
                              p_out->p_buffer, 1, 2);
+drop:
+    block_Release (p_block);
     return p_out;
 }
