@@ -2,7 +2,7 @@
  * demuxdump.c : Pseudo demux module for vlc (dump raw stream)
  *****************************************************************************
  * Copyright (C) 2001-2004 the VideoLAN team
- * $Id: 5f2a5f01b14711d9a0a965453a04a32e53529527 $
+ * $Id$
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -49,20 +49,19 @@
 static int  Open( vlc_object_t * );
 static void Close ( vlc_object_t * );
 
-vlc_module_begin();
-    set_shortname("Dump");
-    set_category( CAT_INPUT );
-    set_subcategory( SUBCAT_INPUT_DEMUX );
-    set_description( N_("File dumper") );
-    set_capability( "demux", 0 );
+vlc_module_begin ()
+    set_shortname("Dump")
+    set_category( CAT_INPUT )
+    set_subcategory( SUBCAT_INPUT_DEMUX )
+    set_description( N_("File dumper") )
+    set_capability( "demux", 0 )
     add_file( "demuxdump-file", "stream-demux.dump", NULL, FILE_TEXT,
-              FILE_LONGTEXT, false );
-        change_unsafe();
+              FILE_LONGTEXT, false )
     add_bool( "demuxdump-append", 0, NULL, APPEND_TEXT, APPEND_LONGTEXT,
-              false );
-    set_callbacks( Open, Close );
-    add_shortcut( "dump" );
-vlc_module_end();
+              false )
+    set_callbacks( Open, Close )
+    add_shortcut( "dump" )
+vlc_module_end ()
 
 
 /*****************************************************************************
@@ -101,6 +100,10 @@ static int Open( vlc_object_t * p_this )
     if( !p_demux->b_force )
         return VLC_EGENERIC;
 
+    p_demux->p_sys = p_sys = malloc( sizeof( demux_sys_t ) );
+    if( !p_sys )
+        return VLC_ENOMEM;
+
     var_Create( p_demux, "demuxdump-append", VLC_VAR_BOOL|VLC_VAR_DOINHERIT );
     var_Get( p_demux, "demuxdump-append", &val );
     b_append = val.b_bool;
@@ -111,13 +114,15 @@ static int Open( vlc_object_t * p_this )
 
     p_demux->pf_demux = Demux;
     p_demux->pf_control = Control;
-    p_demux->p_sys = p_sys = malloc( sizeof( demux_sys_t ) );
+
     p_sys->i_write = 0;
     p_sys->p_file = NULL;
     p_sys->psz_file = var_CreateGetString( p_demux, "demuxdump-file" );
     if( *p_sys->psz_file == '\0' )
     {
         msg_Warn( p_demux, "no dump file name given" );
+        free( p_sys->psz_file );
+        free( p_sys );
         return VLC_EGENERIC;
     }
 
@@ -129,7 +134,7 @@ static int Open( vlc_object_t * p_this )
     else if( ( p_sys->p_file = utf8_fopen( p_sys->psz_file, psz_mode ) ) == NULL )
     {
         msg_Err( p_demux, "cannot create `%s' for writing", p_sys->psz_file );
-
+        free( p_sys->psz_file );
         free( p_sys );
         return VLC_EGENERIC;
     }
@@ -156,7 +161,6 @@ static void Close( vlc_object_t *p_this )
         fclose( p_sys->p_file );
     }
     free( p_sys->psz_file );
-
     free( p_sys );
 }
 
