@@ -2,7 +2,7 @@
  * nsv.c: NullSoft Video demuxer.
  *****************************************************************************
  * Copyright (C) 2004-2007 the VideoLAN team
- * $Id: 19f112019e9148e4d1d0f583fd8f7425ee3b1a16 $
+ * $Id$
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -45,14 +45,14 @@
 static int  Open    ( vlc_object_t * );
 static void Close  ( vlc_object_t * );
 
-vlc_module_begin();
-    set_description( N_("NullSoft demuxer" ) );
-    set_capability( "demux", 10 );
-    set_category( CAT_INPUT );
-    set_subcategory( SUBCAT_INPUT_DEMUX );
-    set_callbacks( Open, Close );
-    add_shortcut( "nsv" );
-vlc_module_end();
+vlc_module_begin ()
+    set_description( N_("NullSoft demuxer" ) )
+    set_capability( "demux", 10 )
+    set_category( CAT_INPUT )
+    set_subcategory( SUBCAT_INPUT_DEMUX )
+    set_callbacks( Open, Close )
+    add_shortcut( "nsv" )
+vlc_module_end ()
 
 /*****************************************************************************
  * Local prototypes
@@ -72,6 +72,8 @@ struct demux_sys_t
     int64_t     i_pcr;
     int64_t     i_time;
     int64_t     i_pcr_inc;
+
+    bool b_start_record;
 };
 
 static int Demux  ( demux_t *p_demux );
@@ -120,6 +122,8 @@ static int Open( vlc_object_t *p_this )
     p_sys->i_time  = 0;
     p_sys->i_pcr_inc = 0;
 
+    p_sys->b_start_record = false;
+
     return VLC_SUCCESS;
 }
 
@@ -159,16 +163,19 @@ static int Demux( demux_t *p_demux )
         if( !memcmp( p_peek, "NSVf", 4 ) )
         {
             if( ReadNSVf( p_demux ) )
-            {
                 return -1;
-            }
         }
         else if( !memcmp( p_peek, "NSVs", 4 ) )
         {
-            if( ReadNSVs( p_demux ) )
+            if( p_sys->b_start_record )
             {
-                return -1;
+                /* Enable recording once synchronized */
+                stream_Control( p_demux->s, STREAM_SET_RECORD_STATE, true, "nsv" );
+                p_sys->b_start_record = false;
             }
+
+            if( ReadNSVs( p_demux ) )
+                return -1;
             break;
         }
         else if( GetWLE( p_peek ) == 0xbeef )
@@ -185,9 +192,7 @@ static int Demux( demux_t *p_demux )
         {
             msg_Err( p_demux, "invalid signature 0x%x (%4.4s)", GetDWLE( p_peek ), (const char*)p_peek );
             if( ReSynch( p_demux ) )
-            {
                 return -1;
-            }
         }
     }
 
@@ -316,6 +321,7 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
     double f, *pf;
+    bool b_bool, *pb_bool;
     int64_t i64, *pi64;
 
     switch( i_query )
@@ -337,11 +343,9 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             f = (double) va_arg( args, double );
             i64 = stream_Size( p_demux->s );
 
-            es_out_Control( p_demux->out, ES_OUT_RESET_PCR );
             if( stream_Seek( p_demux->s, (int64_t)(i64 * f) ) || ReSynch( p_demux ) )
-            {
                 return VLC_EGENERIC;
-            }
+
             p_sys->i_time = -1; /* Invalidate time display */
             return VLC_SUCCESS;
 
@@ -372,6 +376,20 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
             *pf = (double)1000000.0 / (double)p_sys->i_pcr_inc;
             return VLC_SUCCESS;
 
+        case DEMUX_CAN_RECORD:
+            pb_bool = (bool*)va_arg( args, bool * );
+            *pb_bool = true;
+            return VLC_SUCCESS;
+
+        case DEMUX_SET_RECORD_STATE:
+            b_bool = (bool)va_arg( args, int );
+
+            if( !b_bool )
+                stream_Control( p_demux->s, STREAM_SET_RECORD_STATE, false );
+            p_sys->b_start_record = b_bool;
+            return VLC_SUCCESS;
+
+
         case DEMUX_SET_TIME:
         default:
             return VLC_EGENERIC;
@@ -454,9 +472,18 @@ static int ReadNSVs( demux_t *p_demux )
     switch( ( fcc = VLC_FOURCC( header[4], header[5], header[6], header[7] ) ) )
     {
         case VLC_FOURCC( 'V', 'P', '3', ' ' ):
+        case VLC_FOURCC( 'V', 'P', '3', '0' ):
+            fcc = VLC_FOURCC( 'V', 'P', '3', '0' );
+            break;
+
         case VLC_FOURCC( 'V', 'P', '3', '1' ):
             fcc = VLC_FOURCC( 'V', 'P', '3', '1' );
             break;
+
+        case VLC_FOURCC( 'V', 'P', '5', ' ' ):
+        case VLC_FOURCC( 'V', 'P', '5', '0' ):
+            fcc = VLC_FOURCC( 'V', 'P', '5', '0' );
+            break;
         case VLC_FOURCC( 'V', 'P', '6', '0' ):
         case VLC_FOURCC( 'V', 'P', '6', '1' ):
         case VLC_FOURCC( 'V', 'P', '6', '2' ):
@@ -497,6 +524,9 @@ static int ReadNSVs( demux_t *p_demux )
         case VLC_FOURCC( 'A', 'A', 'C', 'P' ):
             fcc = VLC_FOURCC( 'm', 'p', '4', 'a' );
             break;
+        case VLC_FOURCC( 'S', 'P', 'X', ' ' ):
+            fcc = VLC_FOURCC( 's', 'p', 'x', ' ' );
+            break;
         case VLC_FOURCC( 'N', 'O', 'N', 'E' ):
             break;
         default:
