@@ -2,7 +2,7 @@
  * beos.cpp : BeOS plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000, 2001 the VideoLAN team
- * $Id: c1dc7e84b1543a3baef5bffd2bbd497020589af6 $
+ * $Id$
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -49,19 +49,19 @@ void CloseVideo   ( vlc_object_t * );
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-vlc_module_begin();
-    set_category( CAT_INTERFACE );
-    set_subcategory( SUBCAT_INTERFACE_MAIN );
-    add_bool( "beos-dvdmenus", 0, NULL, _("Use DVD Menus"), "", true );
-    set_shortname( "BeOS" );
-    set_description( N_("BeOS standard API interface") );
-    set_capability( "interface", 100 );
-    set_callbacks( OpenIntf, CloseIntf );
+vlc_module_begin ()
+    set_category( CAT_INTERFACE )
+    set_subcategory( SUBCAT_INTERFACE_MAIN )
+    add_bool( "beos-dvdmenus", 0, NULL, _("Use DVD Menus"), "", true )
+    set_shortname( "BeOS" )
+    set_description( N_("BeOS standard API interface") )
+    set_capability( "interface", 100 )
+    set_callbacks( OpenIntf, CloseIntf )
 
-    add_submodule();
-        set_capability( "video output", 100 );
-        set_callbacks( OpenVideo, CloseVideo );
-    add_submodule();
-        set_capability( "audio output", 100 );
-        set_callbacks( OpenAudio, CloseAudio );
-vlc_module_end();
+    add_submodule ()
+        set_capability( "video output", 100 )
+        set_callbacks( OpenVideo, CloseVideo )
+    add_submodule ()
+        set_capability( "audio output", 100 )
+        set_callbacks( OpenAudio, CloseAudio )
+vlc_module_end ()
