@@ -2,7 +2,7 @@
  * controls.h: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2002-2009 the VideoLAN team
- * $Id: 9e07c9faf9266321533b9ff75133b6953ef6f221 $
+ * $Id: 3708813f47af30ff616afb5dd2cd5281359a7f9e $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -57,6 +57,9 @@
 }
 - (void)controlTintChanged;
 
+- (id)voutView;
+- (BOOL)aspectRatioIsLocked;
+
 - (IBAction)play:(id)sender;
 - (IBAction)stop:(id)sender;
 - (IBAction)faster:(id)sender;
@@ -78,10 +81,6 @@
 - (IBAction)forward:(id)sender;
 - (IBAction)backward:(id)sender;
 
-- (id)getVoutView;
-- (BOOL)aspectRatioIsLocked;
-- (IBAction)lockVideosAspectRatio:(id)sender;
-
 - (IBAction)volumeUp:(id)sender;
 - (IBAction)volumeDown:(id)sender;
 - (IBAction)mute:(id)sender;
@@ -89,8 +88,14 @@
 
 - (IBAction)showPosition: (id)sender;
 - (IBAction)toogleFullscreen:(id)sender;
-- (BOOL) isFullscreen;
+- (BOOL)isFullscreen;
 - (IBAction)windowAction:(id)sender;
+
+- (IBAction)telxTransparent:(id)sender;
+- (IBAction)telxNavLink:(id)sender;
+- (IBAction)lockVideosAspectRatio:(id)sender;
+- (IBAction)addSubtitleFile:(id)sender;
+
 - (BOOL)keyEvent:(NSEvent *)o_event;
 - (void)scrollWheel: (NSEvent *)theEvent;
 
@@ -108,31 +113,7 @@
 
 - (IBAction)goToSpecificTime:(id)sender;
 
-- (id)getFSPanel;
-
-@end
-
-/*****************************************************************************
- * VLCMenuExt interface
- *****************************************************************************
- * This holds our data for autogenerated menus
- *****************************************************************************/
-@interface VLCMenuExt : NSObject
-{
-    char *psz_name;
-    int i_object_id;
-    vlc_value_t value;
-    int i_type;
-}
-
-- (id)initWithVar: (const char *)_psz_name 
-           Object: (int)i_id
-            Value: (vlc_value_t)val 
-           ofType: (int)_i_type;
-- (char *)name;
-- (int)objectID;
-- (vlc_value_t)value;
-- (int)type;
+- (id)fspanel;
 
 @end
 
