@@ -2,7 +2,7 @@
  * macosx.m: Mac OS X module for vlc
  *****************************************************************************
  * Copyright (C) 2001-2006 the VideoLAN team
- * $Id: 9d4404dcb00fbf846e33096d47151f3a7248d834 $
+ * $Id$
  *
  * Authors: Colin Delacroix <colin@zoy.org>
  *          Eugenio Jarosiewicz <ej0@cise.ufl.edu>
@@ -87,37 +87,37 @@ void CloseVideoGL ( vlc_object_t * );
 #define EQ_KEEP_LONGTEXT N_("By default, VLC keeps the last equalizer settings before " \
                             "termination. This feature can be disabled here.")
 
-vlc_module_begin();
-    set_description( N_("Mac OS X interface") );
-    set_capability( "interface", 200 );
-    set_callbacks( OpenIntf, CloseIntf );
-    set_category( CAT_INTERFACE );
-    set_subcategory( SUBCAT_INTERFACE_MAIN );
+vlc_module_begin ()
+    set_description( N_("Mac OS X interface") )
+    set_capability( "interface", 200 )
+    set_callbacks( OpenIntf, CloseIntf )
+    set_category( CAT_INTERFACE )
+    set_subcategory( SUBCAT_INTERFACE_MAIN )
     add_bool( "macosx-autoplay", 1, NULL, AUTOPLAY_OSX_TEST, AUTOPLAY_OSX_LONGTEXT,
-              false );
+              false )
     add_bool( "macosx-recentitems", 1, NULL, RECENT_ITEMS_TEXT, RECENT_ITEMS_LONGTEXT,
-              false );
+              false )
     add_bool( "macosx-eq-keep", 1, NULL, EQ_KEEP_TEXT, EQ_KEEP_LONGTEXT,
-              false );
+              false )
     add_bool( "macosx-fspanel", 1, NULL, FSPANEL_TEXT, FSPANEL_LONGTEXT,
-              false );
+              false )
 
-    add_submodule();
-        set_description( "Mac OS X OpenGL" );
-        set_capability( "opengl provider", 100 );
-        set_category( CAT_VIDEO);
-        set_subcategory( SUBCAT_VIDEO_VOUT );
-        set_callbacks( OpenVideoGL, CloseVideoGL );
+    add_submodule ()
+        set_description( "Mac OS X OpenGL" )
+        set_capability( "opengl provider", 100 )
+        set_category( CAT_VIDEO)
+        set_subcategory( SUBCAT_VIDEO_VOUT )
+        set_callbacks( OpenVideoGL, CloseVideoGL )
 
         add_integer( "macosx-vdev", 0, NULL, VDEV_TEXT, VDEV_LONGTEXT,
-                     false );
+                     false )
         add_bool( "macosx-stretch", 0, NULL, STRETCH_TEXT, STRETCH_LONGTEXT,
-                  false );
+                  false )
         add_float_with_range( "macosx-opaqueness", 1, 0, 1, NULL,
                               OPAQUENESS_TEXT, OPAQUENESS_LONGTEXT, true );
         add_bool( "macosx-black", 1, NULL, BLACK_TEXT, BLACK_LONGTEXT,
-                  false );
+                  false )
         add_bool( "macosx-background", 0, NULL, BACKGROUND_TEXT, BACKGROUND_LONGTEXT,
-                  false );
-vlc_module_end();
+                  false )
+vlc_module_end ()
 
