@@ -2,7 +2,7 @@
  * output.m: MacOS X Output Dialog
  *****************************************************************************
  * Copyright (C) 2002-2007 the VideoLAN team
- * $Id: e7f3133d0396801a6d29b3ed01fd7f3891443fb7 $
+ * $Id$
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -59,7 +59,7 @@
     o_mrl = [o_mrl_array copy];
 }
 
-- (NSArray *)getMRL
+- (NSArray *)mrl
 {
     return [o_mrl copy];
 }
