@@ -1,7 +1,7 @@
 /*****************************************************************************
  * mediainfo.cpp : Information about an item
  ****************************************************************************
- * Copyright (C) 2006-2007 the VideoLAN team
+ * Copyright (C) 2006-2008 the VideoLAN team
  * $Id$
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
@@ -21,13 +21,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  ******************************************************************************/
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
 #include "dialogs/mediainfo.hpp"
 #include "input_manager.hpp"
-#include "dialogs_provider.hpp"
 
 #include <QTabWidget>
 #include <QGridLayout>
@@ -42,30 +42,26 @@ MediaInfoDialog *MediaInfoDialog::instance = NULL;
    Please be Careful of not breaking one the modes behaviour... */
 
 MediaInfoDialog::MediaInfoDialog( intf_thread_t *_p_intf,
-                                  input_item_t *_p_item,
-                                  bool _mainInput,
-                                  bool _stats ) :
-                                  QVLCFrame( _p_intf ), mainInput(_mainInput),
-                                  stats( _stats )
+                                  input_item_t *p_item ) :
+                                  QVLCFrame( _p_intf )
 {
-    p_item = _p_item;
-    b_cleaned = true;
-    i_runs = 0;
+    isMainInputInfo = ( p_item == NULL );
 
     setWindowTitle( qtr( "Media Information" ) );
 
     /* TabWidgets and Tabs creation */
-    IT = new QTabWidget;
-    MP = new MetaPanel( IT, p_intf );
-    IT->addTab( MP, qtr( "&General" ) );
-    EMP = new ExtraMetaPanel( IT, p_intf );
-    IT->addTab( EMP, qtr( "&Extra Metadata" ) );
-    IP = new InfoPanel( IT, p_intf );
-    IT->addTab( IP, qtr( "&Codec Details" ) );
-    if( stats )
+    infoTabW = new QTabWidget;
+
+    MP = new MetaPanel( infoTabW, p_intf );
+    infoTabW->addTab( MP, qtr( "&General" ) );
+    EMP = new ExtraMetaPanel( infoTabW, p_intf );
+    infoTabW->addTab( EMP, qtr( "&Extra Metadata" ) );
+    IP = new InfoPanel( infoTabW, p_intf );
+    infoTabW->addTab( IP, qtr( "&Codec Details" ) );
+    if( isMainInputInfo )
     {
-        ISP = new InputStatsPanel( IT, p_intf );
-        IT->addTab( ISP, qtr( "&Statistics" ) );
+        ISP = new InputStatsPanel( infoTabW, p_intf );
+        infoTabW->addTab( ISP, qtr( "&Statistics" ) );
     }
 
     QGridLayout *layout = new QGridLayout( this );
@@ -76,10 +72,10 @@ MediaInfoDialog::MediaInfoDialog( intf_thread_t *_p_intf,
     QPushButton *closeButton = new QPushButton( qtr( "&Close" ) );
     closeButton->setDefault( true );
 
-    uriLine = new QLineEdit;
     QLabel *uriLabel = new QLabel( qtr( "Location:" ) );
+    QLineEdit *uriLine = new QLineEdit;
 
-    layout->addWidget( IT, 0, 0, 1, 8 );
+    layout->addWidget( infoTabW, 0, 0, 1, 8 );
     layout->addWidget( uriLabel, 1, 0, 1, 1 );
     layout->addWidget( uriLine, 1, 1, 1, 7 );
     layout->addWidget( saveMetaButton, 2, 6 );
@@ -91,28 +87,38 @@ MediaInfoDialog::MediaInfoDialog( intf_thread_t *_p_intf,
     BUTTONACT( saveMetaButton, saveMeta() );
 
     /* Let the MetaData Panel update the URI */
-    CONNECT( MP, uriSet( QString ), uriLine, setText( QString ) );
-    CONNECT( MP, editing(), this, showMetaSaveButton() );
+    CONNECT( MP, uriSet( const QString& ), uriLine, setText( const QString& ) );
+    CONNECT( MP, editing(), saveMetaButton, show() );
 
-    CONNECT( IT, currentChanged( int ), this, updateButtons( int ) );
+    /* Display the buttonBar according to the Tab selected */
+    CONNECT( infoTabW, currentChanged( int ), this, updateButtons( int ) );
 
     /* If using the General Mode */
-    if( !p_item )
+    if( isMainInputInfo )
     {
-        msg_Dbg( p_intf, "Using a general windows" );
-        CONNECT( THEMIM, inputChanged( input_thread_t * ),
-                 this, update( input_thread_t * ) );
+        msg_Dbg( p_intf, "Using a general info windows" );
+        /**
+         * Connects on the various signals of input_Manager
+         * For the currently playing element
+         **/
+        CONNECT( THEMIM->getIM(), infoChanged( input_item_t* ),
+                 IP, update( input_item_t* ) );
+        CONNECT( THEMIM->getIM(), metaChanged( input_item_t* ),
+                 MP, update( input_item_t* ) );
+        CONNECT( THEMIM->getIM(), metaChanged( input_item_t* ),
+                 EMP, update( input_item_t* ) );
+        CONNECT( THEMIM->getIM(), statisticsUpdated( input_item_t* ),
+                 ISP, update( input_item_t* ) );
 
         if( THEMIM->getInput() )
             p_item = input_GetItem( THEMIM->getInput() );
     }
+    else
+        msg_Dbg( p_intf, "Using an item specific info windows" );
 
-    /* Call update by hand, so info is shown from current item too */
+    /* Call update at start, so info is filled up at begginning */
     if( p_item )
-        update( p_item, true, true );
-
-    if( stats )
-        ON_TIMEOUT( updateOnTimeOut() );
+        updateAllTabs( p_item );
 
     readSettings( "Mediainfo", QSize( 600 , 480 ) );
 }
@@ -124,86 +130,37 @@ MediaInfoDialog::~MediaInfoDialog()
 
 void MediaInfoDialog::showTab( int i_tab = 0 )
 {
-    IT->setCurrentIndex( i_tab );
+    infoTabW->setCurrentIndex( i_tab );
     show();
 }
 
-void MediaInfoDialog::showMetaSaveButton()
-{
-    saveMetaButton->show();
-}
-
 void MediaInfoDialog::saveMeta()
 {
     MP->saveMeta();
     saveMetaButton->hide();
 }
 
-/* Function called on inputChanged-update*/
-void MediaInfoDialog::update( input_thread_t *p_input )
+void MediaInfoDialog::updateAllTabs( input_item_t *p_item )
 {
-    if( !p_input || p_input->b_dead )
-    {
-        if( !b_cleaned )
-        {
-            clear();
-            b_cleaned = true;
-        }
-        return;
-    }
+    IP->update( p_item );
+    MP->update( p_item );
+    EMP->update( p_item );
 
-    /* Launch the update in all the panels */
-    vlc_object_yield( p_input );
-
-    update( input_GetItem(p_input), true, true);
-
-    vlc_object_release( p_input );
+    if( isMainInputInfo ) ISP->update( p_item );
 }
 
-void MediaInfoDialog::updateOnTimeOut()
-{
-    /* Timer runs at 150 ms, dont' update more than 2 times per second */
-    i_runs++;
-    if( i_runs % 4 != 0 ) return;
-
-    /* Get Input and clear if non-existant */
-    input_thread_t *p_input = THEMIM->getInput();
-
-    if( p_input && !p_input->b_dead )
-    {
-        vlc_object_yield( p_input );
-        update( input_GetItem(p_input), false, false);
-        vlc_object_release( p_input );
-    }
-}
-
-void MediaInfoDialog::update( input_item_t *p_item,
-                              bool update_info,
-                              bool update_meta )
-{
-    if( update_info )
-        IP->update( p_item );
-    if( update_meta )
-    {
-        MP->update( p_item );
-        EMP->update( p_item );
-    }
-    if( stats )
-        ISP->update( p_item );
-}
-
-void MediaInfoDialog::clear()
+void MediaInfoDialog::clearAllTabs()
 {
     IP->clear();
     MP->clear();
     EMP->clear();
-    if( stats ) ISP->clear();
-    b_cleaned = true;
+
+    if( isMainInputInfo ) ISP->clear();
 }
 
 void MediaInfoDialog::close()
 {
-    toggleVisible();
+    hide();
 
     /* if dialog is closed, revert editing if not saved */
     if( MP->isInEditMode() )
@@ -211,9 +168,9 @@ void MediaInfoDialog::close()
         MP->setEditMode( false );
         updateButtons( 0 );
     }
-    if( mainInput == false ) {
+
+    if( !isMainInputInfo )
         deleteLater();
-    }
 }
 
 void MediaInfoDialog::updateButtons( int i_tab )
@@ -223,3 +180,4 @@ void MediaInfoDialog::updateButtons( int i_tab )
     else
         saveMetaButton->hide();
 }
+
