@@ -2,7 +2,7 @@
  * mediainfo.hpp : Information about a stream
  ****************************************************************************
  * Copyright (C) 2006-2007 the VideoLAN team
- * $Id: a9ee843af90e084e5768617629ddb6cc70727ba9 $
+ * $Id$
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -22,75 +22,60 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  ******************************************************************************/
 
-#ifndef _MEDIAINFO_DIALOG_H_
-#define _MEDIAINFO_DIALOG_H_
+#ifndef QVLC_MEDIAINFO_DIALOG_H_
+#define QVLC_MEDIAINFO_DIALOG_H_ 1
 
 #include "util/qvlcframe.hpp"
 #include "components/info_panels.hpp"
 
 class QTabWidget;
-class InfoTab;
-class QLineEdit;
 
 class MediaInfoDialog : public QVLCFrame
 {
     Q_OBJECT;
 public:
     MediaInfoDialog( intf_thread_t *,
-                     input_item_t *,
-                     bool stats = true,
-                     bool mainInput = false );
+                     input_item_t * );
 
     static MediaInfoDialog * getInstance( intf_thread_t *p_intf )
     {
-        if( !instance) instance = new MediaInfoDialog( p_intf,
-                                                       NULL,
-                                                       true,
-                                                       true );
+        if( !instance) instance = new MediaInfoDialog( p_intf, NULL );
         return instance;
     }
 
     static void killInstance()
     {
-        if( instance ) delete instance;
-        instance= NULL;
+        delete instance;
+        instance = NULL;
     }
 
-    virtual ~MediaInfoDialog();
-
     void showTab( int );
 #if 0
     void setInput( input_item_t * );
 #endif
 
 private:
-    input_item_t *p_item;
+    virtual ~MediaInfoDialog();
+
     static MediaInfoDialog *instance;
+    bool isMainInputInfo;
 
-    bool mainInput;
-    bool stats;
-    bool b_cleaned;
-    int i_runs;
+    QTabWidget *infoTabW;
 
-    QTabWidget *IT;
     InputStatsPanel *ISP;
     MetaPanel *MP;
     InfoPanel *IP;
     ExtraMetaPanel *EMP;
 
     QPushButton *saveMetaButton;
-    QLineEdit *uriLine;
-
-public slots:
-    void update( input_thread_t * );
-    void update( input_item_t *, bool, bool );
 
 private slots:
-    void updateOnTimeOut();
-    void close();
-    void clear();
+    void updateAllTabs( input_item_t * );
+    void clearAllTabs();
+
+    virtual void close();
+
     void saveMeta();
-    void showMetaSaveButton();
     void updateButtons( int i_tab );
 };
 
