@@ -1,8 +1,8 @@
 /*****************************************************************************
  * dialogs_provider.cpp : Dialog Provider
  *****************************************************************************
- * Copyright (C) 2006-2008 the VideoLAN team
- * $Id: 0182f352b8915ea4ca0a39b205f3adab9aa85a37 $
+ * Copyright (C) 2006-2009 the VideoLAN team
+ * $Id: f624f145dcba0d0add1d2616c8b290156b067fbe $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -21,22 +21,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#include <QEvent>
-#include <QApplication>
-#include <QSignalMapper>
-#include <QFileDialog>
+#include <vlc_intf_strings.h>
 
-#include <vlc_common.h>
 #include "qt4.hpp"
 #include "dialogs_provider.hpp"
-#include "main_interface.hpp"
+#include "input_manager.hpp" /* Load Subtitles */
 #include "menus.hpp"
-#include <vlc_intf_strings.h>
-#include "input_manager.hpp"
+#include "recents.hpp"
+#include "util/qt_dirs.hpp"
 
 /* The dialogs */
 #include "dialogs/playlist.hpp"
@@ -47,10 +44,21 @@
 #include "dialogs/extended.hpp"
 #include "dialogs/vlm.hpp"
 #include "dialogs/sout.hpp"
+#include "dialogs/convert.hpp"
 #include "dialogs/open.hpp"
+#include "dialogs/openurl.hpp"
 #include "dialogs/help.hpp"
 #include "dialogs/gototime.hpp"
 #include "dialogs/podcast_configuration.hpp"
+#include "dialogs/toolbar.hpp"
+#include "dialogs/plugins.hpp"
+#include "dialogs/external.hpp"
+
+#include <QEvent>
+#include <QApplication>
+#include <QSignalMapper>
+#include <QFileDialog>
+
 
 DialogsProvider* DialogsProvider::instance = NULL;
 
@@ -58,9 +66,8 @@ DialogsProvider::DialogsProvider( intf_thread_t *_p_intf ) :
                                   QObject( NULL ), p_intf( _p_intf )
 {
     b_isDying = false;
-    fixed_timer = new QTimer( this );
-    fixed_timer->start( 150 /* milliseconds */ );
 
+    /* Various signal mappers for the menus */
     menusMapper = new QSignalMapper();
     CONNECT( menusMapper, mapped(QObject *), this, menuAction( QObject *) );
 
@@ -70,11 +77,12 @@ DialogsProvider::DialogsProvider( intf_thread_t *_p_intf ) :
 
     SDMapper = new QSignalMapper();
     CONNECT( SDMapper, mapped (QString), this, SDMenuAction( QString ) );
+
+    DialogHandler *dialogHandler = new DialogHandler (p_intf, this );
 }
 
 DialogsProvider::~DialogsProvider()
 {
-    msg_Dbg( p_intf, "Destroying the Dialog Provider" );
     PlaylistDialog::killInstance();
     MediaInfoDialog::killInstance();
     MessagesDialog::killInstance();
@@ -85,7 +93,6 @@ DialogsProvider::~DialogsProvider()
     UpdateDialog::killInstance();
 #endif
 
-    fixed_timer->stop();
     delete menusMapper;
     delete menusUpdateMapper;
     delete SDMapper;
@@ -93,17 +100,13 @@ DialogsProvider::~DialogsProvider()
 
 void DialogsProvider::quit()
 {
-    /* Stop the playlist */
-    playlist_Stop( THEPL );
     b_isDying = true;
-    vlc_object_kill( p_intf->p_libvlc );
-    QApplication::closeAllWindows();
-    QApplication::quit();
+    libvlc_Quit( p_intf->p_libvlc );
 }
 
 void DialogsProvider::customEvent( QEvent *event )
 {
-    if( event->type() == DialogEvent_Type )
+    if( event->type() == (int)DialogEvent_Type )
     {
         DialogEvent *de = static_cast<DialogEvent*>(event);
         switch( de->i_dialog )
@@ -138,8 +141,6 @@ void DialogsProvider::customEvent( QEvent *event )
         case INTF_DIALOG_VLM:
            vlmDialog(); break;
 #endif
-        case INTF_DIALOG_INTERACTION:
-           doInteraction( de->p_arg ); break;
         case INTF_DIALOG_POPUPMENU:
            QVLCMenu::PopupMenu( p_intf, (de->i_arg != 0) ); break;
         case INTF_DIALOG_AUDIOPOPUPMENU:
@@ -150,7 +151,7 @@ void DialogsProvider::customEvent( QEvent *event )
            QVLCMenu::MiscPopupMenu( p_intf ); break;
         case INTF_DIALOG_WIZARD:
         case INTF_DIALOG_STREAMWIZARD:
-            openThenStreamingDialogs(); break;
+            openAndStreamingDialogs(); break;
 #ifdef UPDATE_CHECK
         case INTF_DIALOG_UPDATEVLC:
             updateDialog(); break;
@@ -173,12 +174,26 @@ void DialogsProvider::playlistDialog()
 
 void DialogsProvider::prefsDialog()
 {
-    PrefsDialog::getInstance( p_intf )->toggleVisible();
+    PrefsDialog *p = new PrefsDialog( (QWidget *)p_intf->p_sys->p_mi, p_intf );
+    p->toggleVisible();
 }
 
 void DialogsProvider::extendedDialog()
 {
-    ExtendedDialog::getInstance( p_intf )->toggleVisible();
+    if( !ExtendedDialog::getInstance( p_intf )->isVisible() || /* Hidden */
+        ExtendedDialog::getInstance( p_intf )->currentTab() != 0 )  /* wrong tab */
+        ExtendedDialog::getInstance( p_intf )->showTab( 0 );
+    else
+        ExtendedDialog::getInstance( p_intf )->hide();
+}
+
+void DialogsProvider::synchroDialog()
+{
+    if( !ExtendedDialog::getInstance( p_intf )->isVisible() || /* Hidden */
+        ExtendedDialog::getInstance( p_intf )->currentTab() != 2 )  /* wrong tab */
+        ExtendedDialog::getInstance( p_intf )->showTab( 2 );
+    else
+        ExtendedDialog::getInstance( p_intf )->hide();
 }
 
 void DialogsProvider::messagesDialog()
@@ -217,7 +232,7 @@ void DialogsProvider::aboutDialog()
 
 void DialogsProvider::mediaInfoDialog()
 {
-    MediaInfoDialog::getInstance( p_intf )->toggleVisible();
+    MediaInfoDialog::getInstance( p_intf )->showTab( 0 );
 }
 
 void DialogsProvider::mediaCodecDialog()
@@ -235,20 +250,19 @@ void DialogsProvider::podcastConfigureDialog()
     PodcastConfigDialog::getInstance( p_intf )->toggleVisible();
 }
 
-
-/****************************************************************************
- * All the open/add stuff
- * Open Dialog first - Simple Open then
- ****************************************************************************/
-
-void DialogsProvider::openDialog( int i_tab )
+void DialogsProvider::toolbarDialog()
 {
-    OpenDialog::getInstance( p_intf->p_sys->p_mi , p_intf )->showTab( i_tab );
+    ToolbarEditDialog *toolbarEditor = new ToolbarEditDialog( (QWidget *)p_intf->p_sys->p_mi, p_intf );
+    if( toolbarEditor->exec() == QDialog::Accepted )
+        emit toolBarConfUpdated();
 }
-void DialogsProvider::openDialog()
+
+void DialogsProvider::pluginDialog()
 {
-    openDialog( OPEN_FILE_TAB );
+    PluginDialog::getInstance( p_intf )->toggleVisible();
 }
+
+/* Generic open file */
 void DialogsProvider::openFileGenericDialog( intf_dialog_args_t *p_arg )
 {
     if( p_arg == NULL )
@@ -274,7 +288,7 @@ void DialogsProvider::openFileGenericDialog( intf_dialog_args_t *p_arg )
     if( p_arg->b_save )
     {
         QString file = QFileDialog::getSaveFileName( NULL, p_arg->psz_title,
-                            qfu( p_intf->p_sys->psz_filepath ), extensions );
+                                        p_intf->p_sys->filepath, extensions );
         if( !file.isEmpty() )
         {
             p_arg->i_results = 1;
@@ -287,13 +301,14 @@ void DialogsProvider::openFileGenericDialog( intf_dialog_args_t *p_arg )
     else /* non-save mode */
     {
         QStringList files = QFileDialog::getOpenFileNames( NULL,
-                p_arg->psz_title, qfu( p_intf->p_sys->psz_filepath ),
+                p_arg->psz_title, p_intf->p_sys->filepath,
                 extensions );
         p_arg->i_results = files.count();
         p_arg->psz_results = (char **)malloc( p_arg->i_results * sizeof( char * ) );
         i = 0;
-        foreach( QString file, files )
+        foreach( const QString &file, files )
             p_arg->psz_results[i++] = strdup( qtu( toNativeSepNoSlash( file ) ) );
+        p_intf->p_sys->filepath = qfu( p_arg->psz_results[i] );
     }
 
     /* Callback */
@@ -311,7 +326,19 @@ void DialogsProvider::openFileGenericDialog( intf_dialog_args_t *p_arg )
     free( p_arg->psz_extensions );
     free( p_arg );
 }
+/****************************************************************************
+ * All the open/add stuff
+ * Open Dialog first - Simple Open then
+ ****************************************************************************/
 
+void DialogsProvider::openDialog( int i_tab )
+{
+    OpenDialog::getInstance( p_intf->p_sys->p_mi , p_intf )->showTab( i_tab );
+}
+void DialogsProvider::openDialog()
+{
+    openDialog( OPEN_FILE_TAB );
+}
 void DialogsProvider::openFileDialog()
 {
     openDialog( OPEN_FILE_TAB );
@@ -332,8 +359,8 @@ void DialogsProvider::openCaptureDialog()
 /* Same as the open one, but force the enqueue */
 void DialogsProvider::PLAppendDialog()
 {
-    OpenDialog::getInstance( p_intf->p_sys->p_mi, p_intf, false, OPEN_AND_ENQUEUE)
-                            ->showTab( OPEN_FILE_TAB );
+    OpenDialog::getInstance( p_intf->p_sys->p_mi, p_intf, false,
+                             OPEN_AND_ENQUEUE )->showTab( OPEN_FILE_TAB );
 }
 
 void DialogsProvider::MLAppendDialog()
@@ -369,10 +396,14 @@ QStringList DialogsProvider::showSimpleOpen( QString help,
     ADD_FILTER_ALL( fileTypes );
     fileTypes.replace(QString(";*"), QString(" *"));
 
-    return QFileDialog::getOpenFileNames( NULL,
+    QStringList files = QFileDialog::getOpenFileNames( NULL,
         help.isEmpty() ? qtr(I_OP_SEL_FILES ) : help,
-        path.isEmpty() ? qfu( p_intf->p_sys->psz_filepath ) : path,
+        path.isEmpty() ? p_intf->p_sys->filepath : path,
         fileTypes );
+
+    if( !files.isEmpty() ) savedirpathFromFile( files.last() );
+
+    return files;
 }
 
 /**
@@ -384,7 +415,8 @@ void DialogsProvider::addFromSimple( bool pl, bool go)
 {
     QStringList files = DialogsProvider::showSimpleOpen();
     int i = 0;
-    foreach( QString file, files )
+    files.sort();
+    foreach( const QString &file, files )
     {
         playlist_Add( THEPL, qtu( toNativeSeparators( file ) ), NULL,
                       go ? ( PLAYLIST_APPEND | ( i ? 0 : PLAYLIST_GO ) |
@@ -392,6 +424,8 @@ void DialogsProvider::addFromSimple( bool pl, bool go)
                          : ( PLAYLIST_APPEND | PLAYLIST_PREPARSE ),
                       PLAYLIST_END,
                       pl ? true : false, false );
+        RecentsMRL::getInstance( p_intf )->addRecent(
+                toNativeSeparators( file ) );
         i++;
     }
 }
@@ -411,6 +445,30 @@ void DialogsProvider::simpleMLAppendDialog()
     addFromSimple( false, false );
 }
 
+/* Url & Clipboard */
+/**
+ * Open a MRL.
+ * If the clipboard contains URLs, the first is automatically 'preselected'.
+ **/
+void DialogsProvider::openUrlDialog()
+{
+    OpenUrlDialog *oud = OpenUrlDialog::getInstance( p_intf->p_sys->p_mi,
+                                                     p_intf );
+    if( oud->exec() == QDialog::Accepted )
+    {
+        QString url = oud->url();
+        if( !url.isEmpty() )
+        {
+            playlist_Add( THEPL, qtu( url ),
+                          NULL, !oud->shouldEnqueue() ?
+                                  ( PLAYLIST_APPEND | PLAYLIST_GO )
+                                : ( PLAYLIST_APPEND | PLAYLIST_PREPARSE ),
+                          PLAYLIST_END, true, false );
+            RecentsMRL::getInstance( p_intf )->addRecent( url );
+        }
+    }
+}
+
 /* Directory */
 /**
  * Open a directory,
@@ -419,24 +477,19 @@ void DialogsProvider::simpleMLAppendDialog()
  **/
 static void openDirectory( intf_thread_t *p_intf, bool pl, bool go )
 {
-    QString dir = QFileDialog::getExistingDirectory( NULL, qtr("Open Directory") );
+    QString dir = QFileDialog::getExistingDirectory( NULL, qtr("Open Directory"), p_intf->p_sys->filepath );
 
     if (!dir.isEmpty() )
     {
-        QString dir2 ;
-        if( dir.endsWith( "VIDEO_TS", Qt::CaseInsensitive ) )
-            dir2 = "dvd://" + dir;
-        else
-            dir2 = "directory://" + dir;
-
-        msg_Dbg( p_intf, "Directory opening: %s", qtu( dir2 ) );
-        input_item_t *p_input = input_item_NewExt( THEPL, qtu( dir2 ),
-                              NULL, 0, NULL, -1 );
+        QString mrl = dir.endsWith( "VIDEO_TS", Qt::CaseInsensitive ) ?
+            "dvd://" : "directory://" + toNativeSeparators( dir );
+        input_item_t *p_input = input_item_New( THEPL, qtu( mrl ), NULL );
 
         /* FIXME: playlist_AddInput() can fail */
         playlist_AddInput( THEPL, p_input,
                        go ? ( PLAYLIST_APPEND | PLAYLIST_GO ) : PLAYLIST_APPEND,
                        PLAYLIST_END, pl, pl_Unlocked );
+        RecentsMRL::getInstance( p_intf )->addRecent( mrl );
         if( !go )
             input_Read( THEPL, p_input, true );
         vlc_gc_decref( p_input );
@@ -463,9 +516,9 @@ void DialogsProvider::MLAppendDir()
  ****************/
 void DialogsProvider::openAPlaylist()
 {
-    QStringList files = showSimpleOpen( qtr( "Open playlist file" ),
+    QStringList files = showSimpleOpen( qtr( "Open playlist..." ),
                                         EXT_FILTER_PLAYLIST );
-    foreach( QString file, files )
+    foreach( const QString &file, files )
     {
         playlist_Import( THEPL, qtu( toNativeSeparators( file ) ) );
     }
@@ -473,96 +526,163 @@ void DialogsProvider::openAPlaylist()
 
 void DialogsProvider::saveAPlaylist()
 {
-    QFileDialog *qfd = new QFileDialog( NULL,
-                                   qtr( "Choose a filename to save playlist" ),
-                                   qfu( p_intf->p_sys->psz_filepath ),
-                                   qtr( "XSPF playlist (*.xspf);; " ) +
-                                   qtr( "M3U playlist (*.m3u);; Any (*.*) " ) );
-    qfd->setFileMode( QFileDialog::AnyFile );
-    qfd->setAcceptMode( QFileDialog::AcceptSave );
-    qfd->setConfirmOverwrite( true );
+    QString file = QFileDialog::getSaveFileName( NULL,
+                                  qtr( "Save playlist as..." ),
+                                  p_intf->p_sys->filepath,
+                                  qtr( "XSPF playlist (*.xspf);; " ) +
+                                  qtr( "M3U playlist (*.m3u);; " ) +
+                                  qtr( "HTML playlist (*.html)" ) );
 
-    if( qfd->exec() == QDialog::Accepted )
+    if( !file.isEmpty() )
     {
-        if( qfd->selectedFiles().count() > 0 )
+        static const char psz_xspf[] = "export-xspf",
+                          psz_m3u[] = "export-m3u",
+                          psz_html[] = "export-html";
+        const char *psz_module;
+
+        if( file.contains( ".xsp" ) )
+            psz_module = psz_xspf;
+        else if( file.contains( ".m3u" ) )
+            psz_module = psz_m3u;
+        else if( file.contains(".html" ) )
+            psz_module = psz_html;
+        else
         {
-            static const char psz_xspf[] = "export-xspf",
-                              psz_m3u[] = "export-m3u";
-            const char *psz_module;
-
-            QString file = qfd->selectedFiles().first();
-            QString filter = qfd->selectedFilter();
-
-            if( file.contains( ".xsp" ) ||
-                ( filter.contains( ".xspf" ) && !file.contains( ".m3u" ) ) )
-            {
-                psz_module = psz_xspf;
-                if( !file.contains( ".xsp" ) )
-                    file.append( ".xspf" );
-            }
-            else
-            {
-                psz_module = psz_m3u;
-                if( !file.contains( ".m3u" ) )
-                    file.append( ".m3u" );
-            }
-
-            playlist_Export( THEPL, qtu( toNativeSeparators( file ) ),
-                        THEPL->p_local_category, psz_module);
+            msg_Warn( p_intf, "Impossible to recognise the file type. "
+                    "Defaulting to XSPF" );
+            psz_module = psz_xspf;
+            file.append( ".xpsf" );
         }
-    }
-    delete qfd;
-}
 
+        playlist_Export( THEPL, qtu( toNativeSeparators( file ) ),
+                THEPL->p_local_category, psz_module);
+    }
+}
 
 /****************************************************************************
  * Sout emulation
  ****************************************************************************/
 
-void DialogsProvider::streamingDialog( QWidget *parent, QString mrl,
-                                       bool b_transcode_only )
+void DialogsProvider::streamingDialog( QWidget *parent,
+                                       QString mrl,
+                                       bool b_transcode_only,
+                                       QStringList options )
 {
-    SoutDialog *s = SoutDialog::getInstance( parent, p_intf, b_transcode_only );
+    char *psz_soutoption;
 
-    if( s->exec() == QDialog::Accepted )
+    /* Stream */
+    if( !b_transcode_only )
     {
-        msg_Dbg( p_intf, "Sout mrl %s", qta( s->getMrl() ) );
-        /* Just do it */
-        int i_len = strlen( qtu( s->getMrl() ) ) + 10;
-        char *psz_option = (char*)malloc( i_len );
-        snprintf( psz_option, i_len - 1, "%s", qtu( s->getMrl() ) );
-
-        playlist_AddExt( THEPL, qtu( mrl ), "Streaming",
-                         PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END,
-                        -1, &psz_option, 1, true, pl_Unlocked );
+        SoutDialog *s = new SoutDialog( parent, p_intf, mrl );
+        if( s->exec() == QDialog::Accepted )
+        {
+            psz_soutoption = strdup( qtu( s->getMrl() ) );
+            delete s;
+        }
+        else
+        {
+            delete s; return;
+        }
+    } else {
+    /* Convert */
+        ConvertDialog *s = new ConvertDialog( parent, p_intf, mrl );
+        if( s->exec() == QDialog::Accepted )
+        {
+            psz_soutoption = strdup( qtu( s->getMrl() ) );
+            delete s;
+        }
+        else
+        {
+            delete s; return;
+        }
     }
+
+    /* Get SoutMRL */
+    if( !EMPTY_STR( psz_soutoption ) )
+    {
+        options += QString( psz_soutoption ).split( " :");
+
+        /* Create Input */
+        input_item_t *p_input;
+        p_input = input_item_New( p_intf, qtu( mrl ), _("Streaming") );
+
+        /* Add normal Options */
+        for( int j = 0; j < options.size(); j++ )
+        {
+            QString qs = colon_unescape( options[j] );
+            if( !qs.isEmpty() )
+            {
+                input_item_AddOption( p_input, qtu( qs ),
+                        VLC_INPUT_OPTION_TRUSTED );
+                msg_Dbg( p_intf, "Adding option: %s", qtu( qs ) );
+            }
+        }
+
+        /* Switch between enqueuing and starting the item */
+        /* FIXME: playlist_AddInput() can fail */
+        playlist_AddInput( THEPL, p_input,
+                PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END, true, pl_Unlocked );
+        vlc_gc_decref( p_input );
+
+        RecentsMRL::getInstance( p_intf )->addRecent( mrl );
+    }
+    free( psz_soutoption );
 }
 
-void DialogsProvider::openThenStreamingDialogs()
+void DialogsProvider::openAndStreamingDialogs()
 {
     OpenDialog::getInstance( p_intf->p_sys->p_mi, p_intf, false, OPEN_AND_STREAM )
                                 ->showTab( OPEN_FILE_TAB );
 }
 
-void DialogsProvider::openThenTranscodingDialogs()
+void DialogsProvider::openAndTranscodingDialogs()
 {
     OpenDialog::getInstance( p_intf->p_sys->p_mi , p_intf, false, OPEN_AND_SAVE )
                                 ->showTab( OPEN_FILE_TAB );
 }
 
+void DialogsProvider::loadSubtitlesFile()
+{
+    input_thread_t *p_input = THEMIM->getInput();
+    if( !p_input ) return;
+
+    input_item_t *p_item = input_GetItem( p_input );
+    if( !p_item ) return;
+
+    char *path = input_item_GetURI( p_item );
+    if( !path ) path = strdup( "" );
+
+    char *sep = strrchr( path, DIR_SEP_CHAR );
+    if( sep ) *sep = '\0';
+
+    QStringList qsl = showSimpleOpen( qtr( "Open subtitles..." ),
+                                      EXT_FILTER_SUBTITLE,
+                                      qfu( path ) );
+    free( path );
+    foreach( const QString &qsFile, qsl )
+    {
+        if( input_AddSubtitle( p_input, qtu( toNativeSeparators( qsFile ) ),
+                    true ) )
+            msg_Warn( p_intf, "unable to load subtitles from '%s'",
+                      qtu( qsFile ) );
+    }
+}
+
+
 /****************************************************************************
- * Menus / Interaction
+ * Menus
  ****************************************************************************/
 
 void DialogsProvider::menuAction( QObject *data )
 {
-    QVLCMenu::DoAction( p_intf, data );
+    QVLCMenu::DoAction( data );
 }
 
 void DialogsProvider::menuUpdateAction( QObject *data )
 {
-    MenuFunc * f = qobject_cast<MenuFunc *>(data);
-    f->doFunc( p_intf );
+    MenuFunc *func = qobject_cast<MenuFunc *>(data);
+    assert( func );
+    func->doFunc( p_intf );
 }
 
 void DialogsProvider::SDMenuAction( QString data )
@@ -575,75 +695,13 @@ void DialogsProvider::SDMenuAction( QString data )
     free( psz_sd );
 }
 
-void DialogsProvider::doInteraction( intf_dialog_args_t *p_arg )
+/**
+ * Play the MRL contained in the Recently played menu.
+ **/
+void DialogsProvider::playMRL( const QString &mrl )
 {
-    InteractionDialog *qdialog;
-    interaction_dialog_t *p_dialog = p_arg->p_dialog;
-    switch( p_dialog->i_action )
-    {
-    case INTERACT_NEW:
-        qdialog = new InteractionDialog( p_intf, p_dialog );
-        p_dialog->p_private = (void*)qdialog;
-        if( !(p_dialog->i_status == ANSWERED_DIALOG) )
-            qdialog->show();
-        break;
-    case INTERACT_UPDATE:
-        qdialog = (InteractionDialog*)(p_dialog->p_private);
-        if( qdialog )
-            qdialog->update();
-        else
-        {
-            /* The INTERACT_NEW message was forgotten
-               so we must create the dialog and update it*/
-            qdialog = new InteractionDialog( p_intf, p_dialog );
-            p_dialog->p_private = (void*)qdialog;
-            if( !(p_dialog->i_status == ANSWERED_DIALOG) )
-                qdialog->show();
-            if( qdialog )
-                qdialog->update();
-        }
-        break;
-    case INTERACT_HIDE:
-        msg_Dbg( p_intf, "Hide the Interaction Dialog" );
-        qdialog = (InteractionDialog*)(p_dialog->p_private);
-        if( qdialog )
-            qdialog->hide();
-        p_dialog->i_status = HIDDEN_DIALOG;
-        break;
-    case INTERACT_DESTROY:
-        msg_Dbg( p_intf, "Destroy the Interaction Dialog" );
-        qdialog = (InteractionDialog*)(p_dialog->p_private);
-        if( !p_dialog->i_flags & DIALOG_NONBLOCKING_ERROR )
-            delete qdialog;
-        p_dialog->i_status = DESTROYED_DIALOG;
-        break;
-    }
-}
+    playlist_Add( THEPL, qtu( mrl ) , NULL,
+           PLAYLIST_APPEND | PLAYLIST_GO , PLAYLIST_END, true, false );
 
-void DialogsProvider::loadSubtitlesFile()
-{
-    input_thread_t *p_input = THEMIM->getInput();
-    if( !p_input )
-        return;
-    input_item_t *p_item = input_GetItem( p_input );
-    if( !p_item )
-        return;
-    char *path = input_item_GetURI( p_item );
-    if( !path )
-        path = strdup( "" );
-    char *sep = strrchr( path, DIR_SEP_CHAR );
-    if( sep )
-        *sep = '\0';
-    QStringList qsl = showSimpleOpen( qtr( "Open subtitles file" ),
-                                      EXT_FILTER_SUBTITLE,
-                                      path );
-    free( path );
-    QString qsFile;
-    foreach( qsFile, qsl )
-    {
-        if( !input_AddSubtitles( p_input, qtu( toNativeSeparators( qsFile ) ),
-                    true ) )
-            msg_Warn( p_intf, "unable to load subtitles from '%s'",
-                      qtu( qsFile ) );
-    }
+    RecentsMRL::getInstance( p_intf )->addRecent( mrl );
 }
