@@ -1,8 +1,8 @@
 /*****************************************************************************
  * xmlparser.hpp
  *****************************************************************************
- * Copyright (C) 2004 VideoLAN
- * $Id: xmlparser.hpp 9417 2004-11-26 10:04:08Z gbazin $
+ * Copyright (C) 2004 the VideoLAN team
+ * $Id$
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *
@@ -18,13 +18,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef XMLPARSER_HPP
 #define XMLPARSER_HPP
 
 #include "../src/skin_common.hpp"
+#include "vlc_block.h"
+#include "vlc_stream.h"
 #include "vlc_xml.h"
 #include <map>
 
@@ -35,7 +37,8 @@
 class XMLParser: public SkinObject
 {
     public:
-        XMLParser( intf_thread_t *pIntf, const string &rFileName );
+        XMLParser( intf_thread_t *pIntf, const string &rFileName,
+                   bool useDTD = true );
         virtual ~XMLParser();
 
         /// Parse the file. Returns true on success
@@ -67,6 +70,7 @@ class XMLParser: public SkinObject
         /// Reader context
         xml_t *m_pXML;
         xml_reader_t *m_pReader;
+        stream_t *m_pStream;
 };
 
 #endif
