@@ -2,7 +2,7 @@
  * theme_loader.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: 03fa1577eca01f17e27eb8e7a52096debcb09bfd $
+ * $Id$
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -116,9 +116,6 @@ bool ThemeLoader::load( const string &fileName )
     }
     free( skin_last );
 
-    // The new theme cannot embed a video output yet
-    VlcProc::instance( getIntf() )->dropVout();
-
     return true;
 }
 
@@ -217,10 +214,7 @@ bool ThemeLoader::extractFileInZip( unzFile file, const string &rootDir )
     // Allocate the buffer
     void *pBuffer = malloc( ZIP_BUFFER_SIZE );
     if( !pBuffer )
-    {
-        msg_Err( getIntf(), "failed to allocate memory" );
         return false;
-    }
 
     // Get the path of the file
     OSFactory *pOsFactory = OSFactory::instance( getIntf() );
@@ -362,10 +356,7 @@ bool ThemeLoader::parse( const string &path, const string &xmlFile )
     // Start the parser
     SkinParser parser( getIntf(), xmlFile, path );
     if( ! parser.parse() )
-    {
-        msg_Err( getIntf(), "failed to parse %s", xmlFile.c_str() );
         return false;
-    }
 
     // Build and store the theme
     Builder builder( getIntf(), parser.getData(), path );
