@@ -2,7 +2,7 @@
  * xosd.c : X On Screen Display interface
  *****************************************************************************
  * Copyright (C) 2001 the VideoLAN team
- * $Id: 0b61f3eb626b544ccf3ef526cc2c942bfb2ed17e $
+ * $Id$
  *
  * Authors: Loïc Minier <lool@videolan.org>
  *
@@ -25,7 +25,6 @@
  * Preamble
  *****************************************************************************/
 
-#include <xosd.h>
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -36,17 +35,17 @@
 #include <vlc_input.h>
 #include <vlc_interface.h>
 
-#ifdef HAVE_UNISTD_H
-#    include <unistd.h>
-#endif
+#include <xosd.h>
 
 /*****************************************************************************
  * intf_sys_t: description and status of rc interface
  *****************************************************************************/
 struct intf_sys_t
 {
-    xosd * p_osd;               /* libxosd handle */
-    bool  b_need_update;   /* Update display ? */
+    xosd *      p_osd;          /* libxosd handle */
+    bool        b_need_update;  /* Update display ? */
+    vlc_mutex_t lock;           /* lock for the condition variable */
+    vlc_cond_t  cond;           /* condition variable to know when to update */
 };
 
 #define MAX_LINE_LENGTH 256
@@ -54,13 +53,13 @@ struct intf_sys_t
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
-static int  Open         ( vlc_object_t * );
-static void Close        ( vlc_object_t * );
+static int  Open        ( vlc_object_t * );
+static void Close       ( vlc_object_t * );
 
-static void Run          ( intf_thread_t * );
+static void Run         ( intf_thread_t * );
 
-static int PlaylistNext( vlc_object_t *p_this, const char *psz_variable,
-                vlc_value_t oval, vlc_value_t nval, void *param );
+static int PlaylistNext ( vlc_object_t *p_this, const char *psz_variable,
+                          vlc_value_t oval, vlc_value_t nval, void *param );
 
 /*****************************************************************************
  * Module descriptor
@@ -83,23 +82,23 @@ static int PlaylistNext( vlc_object_t *p_this, const char *psz_variable,
 #define COLOUR_TEXT N_("Color")
 #define COLOUR_LONGTEXT N_("Color used to display text in the XOSD output.")
 
-vlc_module_begin();
-    set_category( CAT_INTERFACE );
-    set_subcategory( SUBCAT_INTERFACE_CONTROL );
-    set_description( N_("XOSD interface") );
-    set_shortname( "XOSD" );
-    add_bool( "xosd-position", 1, NULL, POSITION_TEXT, POSITION_LONGTEXT, true );
-    add_integer( "xosd-text-offset", 30, NULL, TXT_OFS_TEXT, TXT_OFS_LONGTEXT, true );
+vlc_module_begin ()
+    set_category( CAT_INTERFACE )
+    set_subcategory( SUBCAT_INTERFACE_CONTROL )
+    set_description( N_("XOSD interface") )
+    set_shortname( "XOSD" )
+    add_bool( "xosd-position", 1, NULL, POSITION_TEXT, POSITION_LONGTEXT, true )
+    add_integer( "xosd-text-offset", 30, NULL, TXT_OFS_TEXT, TXT_OFS_LONGTEXT, true )
     add_integer( "xosd-shadow-offset", 2, NULL,
-                 SHD_OFS_TEXT, SHD_OFS_LONGTEXT, true );
+                 SHD_OFS_TEXT, SHD_OFS_LONGTEXT, true )
     add_string( "xosd-font",
                 "-adobe-helvetica-bold-r-normal-*-*-160-*-*-p-*-iso8859-1",
-                NULL, FONT_TEXT, FONT_LONGTEXT, true );
+                NULL, FONT_TEXT, FONT_LONGTEXT, true )
     add_string( "xosd-colour", "LawnGreen",
-                    NULL, COLOUR_TEXT, COLOUR_LONGTEXT, true );
-    set_capability( "interface", 10 );
-    set_callbacks( Open, Close );
-vlc_module_end();
+                    NULL, COLOUR_TEXT, COLOUR_LONGTEXT, true )
+    set_capability( "interface", 10 )
+    set_callbacks( Open, Close )
+vlc_module_end ()
 
 /*****************************************************************************
  * Open: initialize and create stuff
@@ -107,16 +106,9 @@ vlc_module_end();
 static int Open( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
+    intf_sys_t *p_sys;
     xosd *p_osd;
-    char *psz_font, psz_colour;
-
-    /* Allocate instance and initialize some members */
-    p_intf->p_sys = (intf_sys_t *)malloc( sizeof( intf_sys_t ) );
-    if( p_intf->p_sys == NULL )
-    {
-        msg_Err( p_intf, "out of memory" );
-        return VLC_ENOMEM;
-    }
+    char *psz_font, *psz_colour;
 
     if( getenv( "DISPLAY" ) == NULL )
     {
@@ -124,67 +116,57 @@ static int Open( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-    /* Initialize library */
-#if defined(HAVE_XOSD_VERSION_0) || defined(HAVE_XOSD_VERSION_1)
-    psz_font = config_GetPsz( p_intf, "xosd-font" );
-    psz_colour = config_GetPsz( p_intf,"xosd-colour" );
-    p_osd  = p_intf->p_sys->p_osd = xosd_init( psz_font, psz_colour, 3,
-                                               XOSD_top, 0, 1 );
-    free( psz_font );
-    free( psz_colour );
+    /* Allocate instance and initialize some members */
+    p_sys = p_intf->p_sys = malloc( sizeof( intf_sys_t ) );
+    if( p_sys == NULL )
+        return VLC_ENOMEM;
 
-    if( p_intf->p_sys->p_osd == NULL )
-    {
-        msg_Err( p_intf, "couldn't initialize libxosd" );
-        return VLC_EGENERIC;
-    }
-#else
-    p_osd = p_intf->p_sys->p_osd = xosd_create( 1 );
+    /* Initialize library */
+    psz_font = config_GetPsz( p_intf, "xosd-font" );
+    psz_colour = config_GetPsz( p_intf, "xosd-colour" );
+
+    p_osd = xosd_create( 1 );
     if( p_osd == NULL )
     {
         msg_Err( p_intf, "couldn't initialize libxosd" );
+        free( psz_colour );
+        free( psz_font );
+        free( p_sys );
         return VLC_EGENERIC;
     }
-
-    psz_colour = config_GetPsz( p_intf,"xosd-colour" );
-    xosd_set_colour( p_osd, psz_colour );
-    xosd_set_timeout( p_osd, 3 );
-    free( psz_colour );
-#endif
-
-
-    playlist_t *p_playlist = pl_Yield( p_intf );
-    var_AddCallback( p_playlist, "playlist-current", PlaylistNext, p_this );
-    var_AddCallback( p_playlist, "item-change", PlaylistNext, p_this );
-    pl_Release( p_intf );
+    p_sys->p_osd = p_osd;
 
     /* Set user preferences */
-    psz_font = config_GetPsz( p_intf, "xosd-font" );
-    xosd_set_font( p_intf->p_sys->p_osd, psz_font );
-    free( psz_font );
-    xosd_set_outline_colour( p_intf->p_sys->p_osd,"black" );
-#ifdef HAVE_XOSD_VERSION_2
-    xosd_set_horizontal_offset( p_intf->p_sys->p_osd,
+    xosd_set_outline_colour( p_osd, "black" );
+    xosd_set_font( p_osd, psz_font );
+    xosd_set_colour( p_osd, psz_colour );
+    xosd_set_timeout( p_osd, 3 );
+    xosd_set_pos( p_osd, config_GetInt( p_intf, "xosd-position" ) ?
+                                        XOSD_bottom: XOSD_top );
+    xosd_set_horizontal_offset( p_osd,
                     config_GetInt( p_intf, "xosd-text-offset" ) );
-    xosd_set_vertical_offset( p_intf->p_sys->p_osd,
+    xosd_set_vertical_offset( p_osd,
                     config_GetInt( p_intf, "xosd-text-offset" ) );
-#else
-    xosd_set_offset( p_intf->p_sys->p_osd,
-                    config_GetInt( p_intf, "xosd-text-offset" ) );
-#endif
-    xosd_set_shadow_offset( p_intf->p_sys->p_osd,
+    xosd_set_shadow_offset( p_osd,
                     config_GetInt( p_intf, "xosd-shadow-offset" ));
-    xosd_set_pos( p_intf->p_sys->p_osd,
-                    config_GetInt( p_intf, "xosd-position" ) ?
-                                         XOSD_bottom: XOSD_top );
 
     /* Initialize to NULL */
     xosd_display( p_osd, 0, XOSD_string, "XOSD interface initialized" );
 
+    free( psz_colour );
+    free( psz_font );
+
+    // Initialize mutex and condition variable before adding the callbacks
+    vlc_mutex_init( &p_sys->lock );
+    vlc_cond_init( &p_sys->cond );
+    // Add the callbacks
+    playlist_t *p_playlist = pl_Hold( p_intf );
+    var_AddCallback( p_playlist, "item-current", PlaylistNext, p_this );
+    var_AddCallback( p_playlist, "item-change", PlaylistNext, p_this );
+    pl_Release( p_intf );
+
+    p_sys->b_need_update = true;
     p_intf->pf_run = Run;
-
-    p_intf->p_sys->b_need_update = true;
-
     return VLC_SUCCESS;
 }
 
@@ -194,8 +176,9 @@ static int Open( vlc_object_t *p_this )
 static void Close( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
-    playlist_t *p_playlist = pl_Yield( p_intf );
-    var_DelCallback( p_playlist, "playlist-current", PlaylistNext, p_this );
+
+    playlist_t *p_playlist = pl_Hold( p_intf );
+    var_DelCallback( p_playlist, "item-current", PlaylistNext, p_this );
     var_DelCallback( p_playlist, "item-change", PlaylistNext, p_this );
     pl_Release( p_intf );
 
@@ -203,6 +186,8 @@ static void Close( vlc_object_t *p_this )
     xosd_destroy( p_intf->p_sys->p_osd );
 
     /* Destroy structure */
+    vlc_cond_destroy( &p_intf->p_sys->cond );
+    vlc_mutex_destroy( &p_intf->p_sys->lock );
     free( p_intf->p_sys );
 }
 
@@ -215,79 +200,87 @@ static void Run( intf_thread_t *p_intf )
 {
     playlist_t *p_playlist;
     playlist_item_t *p_item = NULL;
-    input_item_t *p_input;
-    char psz_duration[MSTRTIME_MAX_SIZE+2];
     char *psz_display = NULL;
+    int cancel = vlc_savecancel();
 
-    while( vlc_object_alive (p_intf) )
+    while( true )
     {
-        if( p_intf->p_sys->b_need_update == true )
-        {
-            p_intf->p_sys->b_need_update = false;
-            p_playlist = pl_Yield( p_intf );
+        // Wait for a signal
+        vlc_restorecancel( cancel );
+        vlc_mutex_lock( &p_intf->p_sys->lock );
+        mutex_cleanup_push( &p_intf->p_sys->lock );
+        while( !p_intf->p_sys->b_need_update )
+            vlc_cond_wait( &p_intf->p_sys->cond, &p_intf->p_sys->lock );
+        p_intf->p_sys->b_need_update = false;
+        vlc_cleanup_run();
 
-            if( playlist_IsEmpty( p_playlist ) )
+        // Compute the signal
+        cancel = vlc_savecancel();
+        p_playlist = pl_Hold( p_intf );
+        PL_LOCK;
+
+        // If the playlist is empty don't do anything
+        if( playlist_IsEmpty( p_playlist ) )
+        {
+            PL_UNLOCK;
+            pl_Release( p_intf );
+            continue;
+        }
+
+        free( psz_display );
+        int i_status = playlist_Status( p_playlist );
+        if( i_status == PLAYLIST_STOPPED )
+        {
+            psz_display = strdup(_("Stop"));
+        }
+        else if( i_status == PLAYLIST_PAUSED )
+        {
+            psz_display = strdup(_("Pause"));
+        }
+        else
+        {
+            p_item = playlist_CurrentPlayingItem( p_playlist );
+            if( !p_item )
             {
+                psz_display = NULL;
+                PL_UNLOCK;
                 pl_Release( p_intf );
                 continue;
             }
-            free( psz_display );
-            psz_display = NULL;
-            if( p_playlist->status.i_status == PLAYLIST_STOPPED )
+            input_item_t *p_input = p_item->p_input;
+
+            mtime_t i_duration = input_item_GetDuration( p_input );
+            if( i_duration != -1 )
             {
-                psz_display = strdup(_("Stop"));
-                pl_Release( p_intf );
-            }
-            else if( p_playlist->status.i_status == PLAYLIST_PAUSED )
-            {
-                psz_display = strdup(_("Pause"));
-                pl_Release( p_intf );
+                char psz_durationstr[MSTRTIME_MAX_SIZE];
+                secstotimestr( psz_durationstr, i_duration / 1000000 );
+                if( asprintf( &psz_display, "%s (%s)", p_input->psz_name, psz_durationstr ) == -1 )
+                    psz_display = NULL;
             }
             else
-            {
-                p_item = p_playlist->status.p_item;
-                p_input = p_item->p_input;
-
-                pl_Release( p_intf );
-                if( !p_item )
-                    continue;
-
-                mtime_t i_duration = input_item_GetDuration( p_input );
-                if( i_duration != -1 )
-                {
-                    char psz_durationstr[MSTRTIME_MAX_SIZE];
-                    secstotimestr( psz_durationstr, i_duration / 1000000 );
-                    sprintf( psz_duration, "(%s)", psz_durationstr );
-                }
-                else
-                {
-                    sprintf( psz_duration," " );
-                }
-
-                psz_display = (char *)malloc( sizeof(char )*
-                                          (strlen( p_input->psz_name ) +
-                                          MSTRTIME_MAX_SIZE + 2+6 + 10 +10 ));
-                sprintf( psz_display,"%s %s",
-                         p_input->psz_name, psz_duration);
-            }
-
-            /* Display */
-            xosd_display( p_intf->p_sys->p_osd,
-                            0,                               /* first line */
-                            XOSD_string,
-                            psz_display );
+                psz_display = strdup( p_input->psz_name );
         }
+        PL_UNLOCK;
+        pl_Release( p_intf );
 
-        msleep( INTF_IDLE_SLEEP );
+        /* Display */
+        xosd_display( p_intf->p_sys->p_osd, 0, /* first line */
+                      XOSD_string, psz_display );
     }
 }
 
 static int PlaylistNext( vlc_object_t *p_this, const char *psz_variable,
                 vlc_value_t oval, vlc_value_t nval, void *param )
 {
+    (void)p_this;    (void)psz_variable;    (void)oval;    (void)nval;
     intf_thread_t *p_intf = (intf_thread_t *)param;
 
+    // Send the signal using the condition variable
+    vlc_mutex_lock( &p_intf->p_sys->lock );
     p_intf->p_sys->b_need_update = true;
+    vlc_cond_signal( &p_intf->p_sys->cond );
+    vlc_mutex_unlock( &p_intf->p_sys->lock );
+
     return VLC_SUCCESS;
 }
 
