@@ -1,10 +1,10 @@
 /*****************************************************************************
  * export.c :  Playlist export module
  *****************************************************************************
- * Copyright (C) 2004 the VideoLAN team
- * $Id: export.c 11664 2005-07-09 06:17:09Z courmisch $
+ * Copyright (C) 2004-2009 the VideoLAN team
+ * $Id$
  *
- * Authors: Clment Stenac <zorglub@videolan.org>
+ * Authors: Clément Stenac <zorglub@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,38 +18,56 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <vlc/vlc.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
 
 /***************************************************************************
  * Prototypes
  ***************************************************************************/
 int Export_M3U    ( vlc_object_t *p_intf );
 int Export_Old    ( vlc_object_t *p_intf );
-
+int Export_HTML   ( vlc_object_t *p_intf );
+int xspf_export_playlist( vlc_object_t *p_intf );
 
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-vlc_module_begin();
+vlc_module_begin ()
 
-    set_category( CAT_PLAYLIST );
-    set_subcategory( SUBCAT_PLAYLIST_EXPORT );
-    add_submodule();
-        set_description( _("M3U playlist exporter") );
-        add_shortcut( "export-m3u" );
-        set_capability( "playlist export" , 0);
-        set_callbacks( Export_M3U , NULL );
+    set_category( CAT_PLAYLIST )
+    set_subcategory( SUBCAT_PLAYLIST_EXPORT )
+    add_submodule ()
+        set_description( N_("M3U playlist export") )
+        add_shortcut( "export-m3u" )
+        set_capability( "playlist export" , 0 )
+        set_callbacks( Export_M3U , NULL )
 
-    add_submodule();
-        set_description( _("Old playlist exporter") );
-        add_shortcut( "export-old" );
-        set_capability( "playlist export" , 0);
-        set_callbacks( Export_Old , NULL );
+    add_submodule ()
+        set_description( N_("Old playlist export") )
+        add_shortcut( "export-old" )
+        set_capability( "playlist export" , 0 )
+        set_callbacks( Export_Old , NULL )
 
-vlc_module_end();
+    add_submodule ()
+        set_description( N_("XSPF playlist export") )
+        add_shortcut( "export-xspf" )
+        set_capability( "playlist export" , 0 )
+        set_callbacks( xspf_export_playlist , NULL )
+
+    add_submodule ()
+        set_description( N_("HTML playlist export") )
+        add_shortcut( "export-html" )
+        set_capability( "playlist export", 0 )
+        set_callbacks( Export_HTML, NULL )
+
+vlc_module_end ()
