@@ -2,7 +2,7 @@
  * description.c: description stream output module (gathers ES info)
  *****************************************************************************
  * Copyright (C) 2003-2004 the VideoLAN team
- * $Id: description.c 13905 2006-01-12 23:10:04Z dionoea $
+ * $Id$
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -24,12 +24,18 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>
-#include <string.h>
 
-#include <vlc/vlc.h>
-#include <vlc/input.h>
-#include <vlc/sout.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_input.h>
+#include <vlc_block.h>
+#include <vlc_sout.h>
+
+#include <assert.h>
 
 /*****************************************************************************
  * Exported prototypes
@@ -44,22 +50,20 @@ static int               Send( sout_stream_t *, sout_stream_id_t *, block_t* );
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-vlc_module_begin();
-    set_description( _("Description stream output") );
-    set_capability( "sout stream", 50 );
-    add_shortcut( "description" );
-    set_callbacks( Open, Close );
-vlc_module_end();
+vlc_module_begin ()
+    set_description( N_("Description stream output") )
+    set_capability( "sout stream", 50 )
+    add_shortcut( "description" )
+    set_callbacks( Open, Close )
+vlc_module_end ()
 
 struct sout_stream_sys_t
 {
-    input_thread_t *p_input;
     mtime_t i_stream_start;
 };
 
 struct sout_stream_id_t
 {
-    int i_d_u_m_m_y;
 };
 
 /*****************************************************************************
@@ -75,9 +79,6 @@ static int Open( vlc_object_t *p_this )
     p_stream->pf_send = Send;
     p_sys = p_stream->p_sys = malloc(sizeof(sout_stream_sys_t));
 
-    p_sys->p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_PARENT );
-    if( !p_sys->p_input ) return VLC_EGENERIC;
-
     p_sys->i_stream_start = 0;
 
     return VLC_SUCCESS;
@@ -89,32 +90,46 @@ static int Open( vlc_object_t *p_this )
 static void Close( vlc_object_t *p_this )
 {
     sout_stream_t *p_stream = (sout_stream_t *)p_this;
-    vlc_object_release( p_stream->p_sys->p_input );
+    sout_stream_sys_t *p_sys = p_stream->p_sys;
+
+    msg_Dbg( p_this, "Closing" );
+
+    free( p_sys );
 }
 
 static sout_stream_id_t *Add( sout_stream_t *p_stream, es_format_t *p_fmt )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
     sout_stream_id_t *id;
-    es_format_t *p_fmt_copy = malloc(sizeof(es_format_t));
+    es_format_t *p_fmt_copy;
+    input_item_t *p_item;
+    input_thread_t *p_input;
 
-    id = malloc( sizeof( sout_stream_id_t ) );
-    id->i_d_u_m_m_y = 0;
+    msg_Dbg( p_stream, "Adding a stream" );
+    p_input = vlc_object_find( p_stream, VLC_OBJECT_INPUT, FIND_PARENT );
+    assert( p_input );
+ 
+    p_item = input_GetItem(p_input);
 
+    p_fmt_copy = malloc(sizeof(es_format_t));
     es_format_Copy( p_fmt_copy, p_fmt );
 
-    vlc_mutex_lock( &p_sys->p_input->input.p_item->lock );
-    TAB_APPEND( p_sys->p_input->input.p_item->i_es,
-                p_sys->p_input->input.p_item->es, p_fmt_copy );
-    vlc_mutex_unlock( &p_sys->p_input->input.p_item->lock );
+    vlc_mutex_lock( &p_item->lock );
+    TAB_APPEND( p_item->i_es, p_item->es, p_fmt_copy );
+    vlc_mutex_unlock( &p_item->lock );
+    vlc_object_release( p_input );
 
-    if( p_sys->i_stream_start <= 0 ) p_sys->i_stream_start = mdate();
+    if( p_sys->i_stream_start <= 0 )
+        p_sys->i_stream_start = mdate();
 
+    id = malloc( sizeof( sout_stream_id_t ) );
     return id;
 }
 
 static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
 {
+    msg_Dbg( p_stream, "Removing a stream" );
+
     free( id );
     return VLC_SUCCESS;
 }
@@ -122,13 +137,21 @@ static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
 static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
                  block_t *p_buffer )
 {
+    VLC_UNUSED(id);
     sout_stream_sys_t *p_sys = p_stream->p_sys;
 
     block_ChainRelease( p_buffer );
 
     if( p_sys->i_stream_start + 1500000 < mdate() )
     {
-        p_sys->p_input->b_eof = VLC_TRUE;
+        input_thread_t *p_input;
+
+        p_input = vlc_object_find( p_stream, VLC_OBJECT_INPUT, FIND_PARENT );
+        if( p_input )
+        {
+            p_input->b_eof = true;
+            vlc_object_release( p_input );
+        }
     }
 
     return VLC_SUCCESS;
