@@ -2,7 +2,7 @@
  * standard.c: standard stream output module
  *****************************************************************************
  * Copyright (C) 2003-2007 the VideoLAN team
- * $Id: 22a673b7cbd752c28e11a043fcb0208373843a12 $
+ * $Id$
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -96,45 +96,44 @@ static void     Close   ( vlc_object_t * );
 
 #define SOUT_CFG_PREFIX "sout-standard-"
 
-vlc_module_begin();
-    set_shortname( N_("Standard"));
-    set_description( N_("Standard stream output") );
-    set_capability( "sout stream", 50 );
-    add_shortcut( "standard" );
-    add_shortcut( "std" );
-    set_category( CAT_SOUT );
-    set_subcategory( SUBCAT_SOUT_STREAM );
+vlc_module_begin ()
+    set_shortname( N_("Standard"))
+    set_description( N_("Standard stream output") )
+    set_capability( "sout stream", 50 )
+    add_shortcut( "standard" )
+    add_shortcut( "std" )
+    set_category( CAT_SOUT )
+    set_subcategory( SUBCAT_SOUT_STREAM )
 
     add_string( SOUT_CFG_PREFIX "access", "", NULL, ACCESS_TEXT,
-                ACCESS_LONGTEXT, false );
+                ACCESS_LONGTEXT, false )
     add_string( SOUT_CFG_PREFIX "mux", "", NULL, MUX_TEXT,
-                MUX_LONGTEXT, false );
+                MUX_LONGTEXT, false )
     add_string( SOUT_CFG_PREFIX "dst", "", NULL, DEST_TEXT,
-                DEST_LONGTEXT, false );
+                DEST_LONGTEXT, false )
     add_string( SOUT_CFG_PREFIX "bind", "", NULL, BIND_TEXT,
-                BIND_LONGTEXT, false );
+                BIND_LONGTEXT, false )
     add_string( SOUT_CFG_PREFIX "path", "", NULL, PATH_TEXT,
-                PATH_LONGTEXT, false );
-        change_unsafe();
+                PATH_LONGTEXT, false )
 
     add_bool( SOUT_CFG_PREFIX "sap", false, NULL, SAP_TEXT, SAP_LONGTEXT,
-              true );
+              true )
     add_string( SOUT_CFG_PREFIX "name", "", NULL, NAME_TEXT, NAME_LONGTEXT,
-                                        true );
+                                        true )
     add_string( SOUT_CFG_PREFIX "group", "", NULL, GROUP_TEXT, GROUP_LONGTEXT,
-                                        true );
+                                        true )
     add_string( SOUT_CFG_PREFIX "description", "", NULL, DESC_TEXT, DESC_LONGTEXT,
-                                        true );
+                                        true )
     add_string( SOUT_CFG_PREFIX "url", "", NULL, URL_TEXT, URL_LONGTEXT,
-                                        true );
+                                        true )
     add_string( SOUT_CFG_PREFIX "email", "", NULL, EMAIL_TEXT, EMAIL_LONGTEXT,
-                                        true );
+                                        true )
     add_string( SOUT_CFG_PREFIX "phone", "", NULL, PHONE_TEXT, PHONE_LONGTEXT,
-                                        true );
-    add_obsolete_bool( SOUT_CFG_PREFIX "sap-ipv6" );
+                                        true )
+    add_obsolete_bool( SOUT_CFG_PREFIX "sap-ipv6" )
 
-    set_callbacks( Open, Close );
-vlc_module_end();
+    set_callbacks( Open, Close )
+vlc_module_end ()
 
 
 /*****************************************************************************
@@ -465,6 +464,9 @@ static int Open( vlc_object_t *p_this )
     free( psz_mux );
     free( psz_url );
 
+    if( !sout_AccessOutCanControlPace( p_access ) )
+        p_sout->i_out_pace_nocontrol++;
+
     return VLC_SUCCESS;
 }
 
@@ -481,6 +483,8 @@ static void Close( vlc_object_t * p_this )
         sout_AnnounceUnRegister( p_stream->p_sout, p_sys->p_session );
 
     sout_MuxDelete( p_sys->p_mux );
+    if( !sout_AccessOutCanControlPace( p_access ) )
+        p_stream->p_sout->i_out_pace_nocontrol--;
     sout_AccessOutDelete( p_access );
 
     free( p_sys );
