@@ -5,7 +5,7 @@
  * Copyright (C) 2003 Antoine Missout
  * Copyright (C) 2000-2003 Michel Lespinasse <walken@zoy.org>
  * Copyright (C) 1999-2000 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
- * $Id: 67e6332caab2e32f3ca95d3244d56611bda2afbe $
+ * $Id$
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -32,13 +32,13 @@
  * Preamble
  *****************************************************************************/
 #define NDEBUG 1
-#include <assert.h>
-#include <math.h>
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
+#include <assert.h>
+#include <math.h>
+
 #include <vlc_common.h>
 #include <vlc_sout.h>
 #include <vlc_codec.h>
@@ -149,7 +149,7 @@ static const uint8_t map_non_linear_mquant[113] =
 
 int scale_quant( transrate_t *tr, double qrate )
 {
-    int i_quant = (int)floor( tr->quantizer_scale * qrate + 0.5 );
+    int i_quant = floor( tr->quantizer_scale * qrate + 0.5 );
 
     if ( tr->q_scale_type )
     {
@@ -171,7 +171,7 @@ int scale_quant( transrate_t *tr, double qrate )
     return i_quant;
 }
 
-int increment_quant( transrate_t *tr, int i_quant )
+static int increment_quant( transrate_t *tr, int i_quant )
 {
     if ( tr->q_scale_type )
     {
