@@ -84,26 +84,26 @@ static const uint64_t i_80w   = 0x0000000080808080ULL;
 /*****************************************************************************
  * Module descriptor.
  *****************************************************************************/
-vlc_module_begin();
+vlc_module_begin ()
 #if defined (MODULE_NAME_IS_i420_yuy2)
-    set_description( N_("Conversions from " SRC_FOURCC " to " DEST_FOURCC) );
-    set_capability( "video filter2", 80 );
+    set_description( N_("Conversions from " SRC_FOURCC " to " DEST_FOURCC) )
+    set_capability( "video filter2", 80 )
 #elif defined (MODULE_NAME_IS_i420_yuy2_mmx)
-    set_description( N_("MMX conversions from " SRC_FOURCC " to " DEST_FOURCC) );
-    set_capability( "video filter2", 160 );
-    add_requirement( MMX );
+    set_description( N_("MMX conversions from " SRC_FOURCC " to " DEST_FOURCC) )
+    set_capability( "video filter2", 160 )
+    add_requirement( MMX )
 #elif defined (MODULE_NAME_IS_i420_yuy2_sse2)
-    set_description( N_("SSE2 conversions from " SRC_FOURCC " to " DEST_FOURCC) );
-    set_capability( "video filter2", 250 );
-    add_requirement( SSE2 );
+    set_description( N_("SSE2 conversions from " SRC_FOURCC " to " DEST_FOURCC) )
+    set_capability( "video filter2", 250 )
+    add_requirement( SSE2 )
 #elif defined (MODULE_NAME_IS_i420_yuy2_altivec)
     set_description(
             _("AltiVec conversions from " SRC_FOURCC " to " DEST_FOURCC) );
-    set_capability( "video filter2", 250 );
-    add_requirement( ALTIVEC );
+    set_capability( "video filter2", 250 )
+    add_requirement( ALTIVEC )
 #endif
-    set_callbacks( Activate, NULL );
-vlc_module_end();
+    set_callbacks( Activate, NULL )
+vlc_module_end ()
 
 /*****************************************************************************
  * Activate: allocate a chroma function
