@@ -2,7 +2,7 @@
  * chain.c : chain multiple video filter modules as a last resort solution
  *****************************************************************************
  * Copyright (C) 2007-2008 the VideoLAN team
- * $Id: 1c295bba856d842c800c93b2bf8f014177a23150 $
+ * $Id$
  *
  * Authors: Antoine Cellerier <dionoea at videolan dot org>
  *
@@ -39,11 +39,11 @@
 static int       Activate   ( vlc_object_t * );
 static void      Destroy    ( vlc_object_t * );
 
-vlc_module_begin();
-    set_description( N_("Video filtering using a chain of video filter modules") );
-    set_capability( "video filter2", 1 );
-    set_callbacks( Activate, Destroy );
-vlc_module_end();
+vlc_module_begin ()
+    set_description( N_("Video filtering using a chain of video filter modules") )
+    set_capability( "video filter2", 1 )
+    set_callbacks( Activate, Destroy )
+vlc_module_end ()
 
 /*****************************************************************************
  * Local prototypes.
@@ -182,13 +182,13 @@ static int BuildChromaChain( filter_t *p_filter )
 
     if( var_Create( p_filter, MODULE_STRING"-level", VLC_VAR_INTEGER | (b_first ? VLC_VAR_DOINHERIT : 0 ) ) )
     {
-        msg_Err( p_filter, "Failed to create %s variable\n", psz_option );
+        msg_Err( p_filter, "Failed to create %s variable", psz_option );
         return VLC_EGENERIC;
     }
     int i_level = var_GetInteger( p_filter, psz_option );
     if( i_level >= CHAIN_LEVEL_MAX )
     {
-        msg_Err( p_filter, "Too high level of recursion (%d)\n", i_level );
+        msg_Err( p_filter, "Too high level of recursion (%d)", i_level );
         return VLC_EGENERIC;
     }
     var_SetInteger( p_filter, psz_option, i_level + 1 );
