@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout.h: Windows video output header file
  *****************************************************************************
- * Copyright (C) 2001-2004 the VideoLAN team
- * $Id: d964c3630007c5479d34ebada27f0a71d985ce12 $
+ * Copyright (C) 2001-2009 the VideoLAN team
+ * $Id$
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Damien Fouilleul <damienf@videolan.org>
@@ -30,6 +30,7 @@ typedef struct event_thread_t
     VLC_COMMON_MEMBERS
 
     vout_thread_t * p_vout;
+    HANDLE window_ready;
 
 } event_thread_t;
 
@@ -64,8 +65,11 @@ typedef struct event_thread_t
 #       define kfDirect444      0x200
 #       define kfDirectInverted 0x400
 #   endif
+
 #endif
 
+struct vout_window_t;
+
 /*****************************************************************************
  * vout_sys_t: video output method descriptor
  *****************************************************************************
@@ -76,6 +80,7 @@ struct vout_sys_t
 {
     HWND                 hwnd;                  /* Handle of the main window */
     HWND                 hvideownd;        /* Handle of the video sub-window */
+    struct vout_window_t *parent_window;         /* Parent window VLC object */
     HWND                 hparent;             /* Handle of the parent window */
     HWND                 hfswnd;          /* Handle of the fullscreen window */
     WNDPROC              pf_wndproc;             /* Window handling callback */
@@ -194,11 +199,22 @@ struct vout_sys_t
     int        i_depth;
     int        render_width;
     int        render_height;
+	    /* Our offscreen bitmap and its framebuffer */
+    HDC        off_dc;
+    HBITMAP    off_bitmap;
+    uint8_t *  p_pic_buffer;
+    int        i_pic_pitch;
+    int        i_pic_pixel_pitch;
+
+    BITMAPINFO bitmapinfo;
+    RGBQUAD    red;
+    RGBQUAD    green;
+    RGBQUAD    blue;
 
     bool b_focus;
     bool b_parent_focus;
 
-    HINSTANCE  gapi_dll;                    /* handle of the opened gapi dll */
+    HINSTANCE  gapi_dll;                   /* handle of the opened gapi dll */
 
     /* GAPI functions */
     int (*GXOpenDisplay)( HWND hWnd, DWORD dwFlags );
@@ -219,6 +235,16 @@ struct vout_sys_t
     vlc_mutex_t    lock;
 };
 
+#ifdef MODULE_NAME_IS_wingapi
+#   define GXOpenDisplay p_vout->p_sys->GXOpenDisplay
+#   define GXCloseDisplay p_vout->p_sys->GXCloseDisplay
+#   define GXBeginDraw p_vout->p_sys->GXBeginDraw
+#   define GXEndDraw p_vout->p_sys->GXEndDraw
+#   define GXGetDisplayProperties p_vout->p_sys->GXGetDisplayProperties
+#   define GXSuspend p_vout->p_sys->GXSuspend
+#   define GXResume p_vout->p_sys->GXResume
+#endif
+
 /*****************************************************************************
  * Prototypes from directx.c
  *****************************************************************************/
