@@ -1,10 +1,10 @@
 /*****************************************************************************
  * access.c
  *****************************************************************************
- * Copyright (C) 1999-2004 the VideoLAN team
- * $Id: 48bd44eb2a94e1f749bc00999630723985671525 $
+ * Copyright (C) 1999-2008 the VideoLAN team
+ * $Id$
  *
- * Author: Laurent Aimar <fenrir@via.ecp.fr>
+ * Author: Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,36 +25,26 @@
 # include "config.h"
 #endif
 
-#include <vlc_common.h>
-
-#include "input_internal.h"
+#include "access.h"
+#include <libvlc.h>
 
 /*****************************************************************************
- * access_InternalNew:
+ * access_New:
  *****************************************************************************/
-static access_t *access_InternalNew( vlc_object_t *p_obj, const char *psz_access,
-                                      const char *psz_demux, const char *psz_path,
-                                      access_t *p_source )
+access_t *__access_New( vlc_object_t *p_obj, const char *psz_access,
+                        const char *psz_demux, const char *psz_path )
 {
-    static const char typename[] = "access";
     access_t *p_access = vlc_custom_create( p_obj, sizeof (*p_access),
-                                            VLC_OBJECT_GENERIC, typename );
+                                            VLC_OBJECT_GENERIC, "access" );
 
     if( p_access == NULL )
         return NULL;
 
-    /* Parse URL */
-    p_access->p_source = p_source;
-    if( p_source )
-    {
-        msg_Dbg( p_obj, "creating access filter '%s'", psz_access );
-    }
-    else
-    {
-        msg_Dbg( p_obj, "creating access '%s' path='%s'",
-                 psz_access, psz_path );
-        p_access->psz_path   = strdup( psz_path );
-    }
+    /* */
+    msg_Dbg( p_obj, "creating access '%s' path='%s'",
+             psz_access, psz_path );
+
+    p_access->psz_path   = strdup( psz_path );
     p_access->psz_access = strdup( psz_access );
     p_access->psz_demux  = strdup( psz_demux );
 
@@ -63,25 +53,16 @@ static access_t *access_InternalNew( vlc_object_t *p_obj, const char *psz_access
     p_access->pf_seek    = NULL;
     p_access->pf_control = NULL;
     p_access->p_sys      = NULL;
-    p_access->info.i_update = 0;
-    p_access->info.i_size   = 0;
-    p_access->info.i_pos    = 0;
-    p_access->info.b_eof    = false;
-    p_access->info.b_prebuffered = false;
-    p_access->info.i_title  = 0;
-    p_access->info.i_seekpoint = 0;
 
+    access_InitFields( p_access );
 
-    /* Before module_Need (for var_Create...) */
+    /* Before module_need (for var_Create...) */
     vlc_object_attach( p_access, p_obj );
 
-    p_access->p_module =
-         module_Need( p_access, p_source ? "access_filter" : "access",
-                      psz_access, true );
+    p_access->p_module = module_need( p_access, "access", psz_access, true );
 
     if( p_access->p_module == NULL )
     {
-        msg_StackAdd( "could not create access" );
         vlc_object_detach( p_access );
         free( p_access->psz_access );
         free( p_access->psz_path );
@@ -93,43 +74,18 @@ static access_t *access_InternalNew( vlc_object_t *p_obj, const char *psz_access
     return p_access;
 }
 
-/*****************************************************************************
- * access_New:
- *****************************************************************************/
-access_t *__access_New( vlc_object_t *p_obj, const char *psz_access,
-                         const char *psz_demux, const char *psz_path )
-{
-    return access_InternalNew( p_obj, psz_access, psz_demux,
-                                psz_path, NULL );
-}
-
-/*****************************************************************************
- * access_FilterNew:
- *****************************************************************************/
-access_t *access_FilterNew( access_t *p_source, const char *psz_access_filter )
-{
-    return access_InternalNew( VLC_OBJECT(p_source), psz_access_filter,
-                                p_source->psz_demux, p_source->psz_path,
-                                p_source );
-}
-
 /*****************************************************************************
  * access_Delete:
  *****************************************************************************/
 void access_Delete( access_t *p_access )
 {
-    module_Unneed( p_access, p_access->p_module );
+    module_unneed( p_access, p_access->p_module );
     vlc_object_detach( p_access );
 
     free( p_access->psz_access );
     free( p_access->psz_path );
     free( p_access->psz_demux );
 
-    if( p_access->p_source )
-    {
-        access_Delete( p_access->p_source );
-    }
-
     vlc_object_release( p_access );
 }
 
