@@ -29,6 +29,7 @@
 #define _VLM_INTERNAL_H 1
 
 #include <vlc_vlm.h>
+#include "input_internal.h"
 
 /* Private */
 typedef struct
@@ -41,9 +42,9 @@ typedef struct
 
     bool      b_sout_keep;
 
-    input_item_t    *p_item;
-    input_thread_t  *p_input;
-    sout_instance_t *p_sout;
+    input_item_t      *p_item;
+    input_thread_t    *p_input;
+    input_resource_t *p_input_resource;
 
 } vlm_media_instance_sys_t;
 
@@ -87,7 +88,8 @@ struct vlm_t
 {
     VLC_COMMON_MEMBERS
 
-    vlc_mutex_t lock;
+    vlc_mutex_t  lock;
+    vlc_thread_t thread;
 
     /* */
     int64_t        i_id;
