@@ -4,7 +4,7 @@
  * interface, such as command line.
  *****************************************************************************
  * Copyright (C) 1998-2007 the VideoLAN team
- * $Id: c4b031e5fa284eca5f4fc53bab48ef4b24f64559 $
+ * $Id: 9b99c1b62538963816b90f72f0457b2661c05458 $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -43,7 +43,9 @@
 #include <vlc_vout.h>
 
 #include "vlc_interface.h"
-#include "modules/modules.h" // Gruik!
+#if defined( __APPLE__ ) || defined( WIN32 )
+#include "../control/libvlc_internal.h"
+#endif
 #include "libvlc.h"
 
 /*****************************************************************************
@@ -57,35 +59,26 @@ static int AddIntfCallback( vlc_object_t *, char const *,
                             vlc_value_t , vlc_value_t , void * );
 
 /**
- * \brief Destroy the interface after the main loop endeed.
+ * Destroy the interface after the main loop endeed.
  *
- * \param p_intf the interface thread
- * \return nothing
+ * @param p_obj: the interface thread
  */
 static void intf_Destroy( vlc_object_t *obj )
 {
     intf_thread_t *p_intf = (intf_thread_t *)obj;
 
-    /* Unlock module if present (a switch may have failed) */
-    if( p_intf->p_module )
-        module_Unneed( p_intf, p_intf->p_module );
-
     free( p_intf->psz_intf );
-    vlc_mutex_destroy( &p_intf->change_lock );
+    config_ChainDestroy( p_intf->p_cfg );
 }
 
-/*****************************************************************************
- * intf_Create: prepare interface before main loop
- *****************************************************************************
- * This function opens output devices and creates specific interfaces. It sends
- * its own error messages.
- *****************************************************************************/
+
 /**
- * Create the interface, and prepare it for main loop.
+ * Create the interface, and prepare it for main loop. It opens ouput device
+ * and creates specific interfaces. Sends its own error messages.
  *
- * \param p_this the calling vlc_object_t
- * \param psz_module a preferred interface module
- * \return a pointer to the created interface thread, NULL on error
+ * @param p_this the calling vlc_object_t
+ * @param psz_module a preferred interface module
+ * @return a pointer to the created interface thread, NULL on error
  */
 intf_thread_t* __intf_Create( vlc_object_t *p_this, const char *psz_module )
 {
@@ -95,14 +88,20 @@ intf_thread_t* __intf_Create( vlc_object_t *p_this, const char *psz_module )
     p_intf = vlc_object_create( p_this, VLC_OBJECT_INTF );
     if( !p_intf )
         return NULL;
-    p_intf->b_interaction = false;
 #if defined( __APPLE__ ) || defined( WIN32 )
     p_intf->b_should_run_on_first_thread = false;
 #endif
 
     /* Choose the best module */
-    p_intf->psz_intf = strdup( psz_module );
-    p_intf->p_module = module_Need( p_intf, "interface", psz_module, false );
+    p_intf->p_cfg = NULL;
+    char *psz_parser = *psz_module == '$'
+                     ? var_CreateGetString(p_intf,psz_module+1)
+                     : strdup( psz_module );
+    char *psz_tmp = config_ChainCreate( &p_intf->psz_intf, &p_intf->p_cfg,
+                                        psz_parser );
+    free( psz_tmp );
+    free( psz_parser );
+    p_intf->p_module = module_need( p_intf, "interface", p_intf->psz_intf, true );
 
     if( p_intf->p_module == NULL )
     {
@@ -112,13 +111,6 @@ intf_thread_t* __intf_Create( vlc_object_t *p_this, const char *psz_module )
         return NULL;
     }
 
-    /* Initialize structure */
-    p_intf->b_menu        = false;
-    p_intf->b_menu_change = false;
-
-    /* Initialize mutexes */
-    vlc_mutex_init( &p_intf->change_lock );
-
     /* Attach interface to its parent object */
     vlc_object_attach( p_intf, p_this );
     vlc_object_set_destructor( p_intf, intf_Destroy );
@@ -126,16 +118,12 @@ intf_thread_t* __intf_Create( vlc_object_t *p_this, const char *psz_module )
     return p_intf;
 }
 
-/*****************************************************************************
- * intf_RunThread: launch the interface thread
- *****************************************************************************
- * This function either creates a new thread and runs the interface in it.
- *****************************************************************************/
+
 /**
  * Starts and runs the interface thread.
  *
- * \param p_intf the interface thread
- * \return VLC_SUCCESS on success, an error number else
+ * @param p_intf the interface thread
+ * @return VLC_SUCCESS on success, an error number else
  */
 int intf_RunThread( intf_thread_t *p_intf )
 {
@@ -145,27 +133,22 @@ int intf_RunThread( intf_thread_t *p_intf )
     if( p_intf->b_should_run_on_first_thread )
     {
         if( vlc_thread_create( p_intf, "interface", MonitorLibVLCDeath,
-                               VLC_THREAD_PRIORITY_LOW, false ) )
+                               VLC_THREAD_PRIORITY_LOW ) )
         {
             msg_Err( p_intf, "cannot spawn libvlc death monitoring thread" );
             return VLC_EGENERIC;
         }
         RunInterface( VLC_OBJECT(p_intf) );
 
-        /* Make sure our MonitorLibVLCDeath thread exit */
-        vlc_object_kill( p_intf );
         /* It is monitoring libvlc, not the p_intf */
-        vlc_object_signal( p_intf->p_libvlc );
-        vlc_thread_join( p_intf );
+        vlc_object_kill( p_intf->p_libvlc );
 
-        vlc_object_detach( p_intf );
-        vlc_object_release( p_intf );
         return VLC_SUCCESS;
     }
 #endif
     /* Run the interface in a separate thread */
     if( vlc_thread_create( p_intf, "interface", RunInterface,
-                           VLC_THREAD_PRIORITY_LOW, false ) )
+                           VLC_THREAD_PRIORITY_LOW ) )
     {
         msg_Err( p_intf, "cannot spawn interface thread" );
         return VLC_EGENERIC;
@@ -174,30 +157,36 @@ int intf_RunThread( intf_thread_t *p_intf )
     return VLC_SUCCESS;
 }
 
+
 /**
  * Stops the interface thread
  *
  * This function asks the interface thread to stop
- * \param p_intf the interface thread
- * \return nothing
+ * @param p_intf the interface thread
  */
 void intf_StopThread( intf_thread_t *p_intf )
 {
     /* Tell the interface to die */
     vlc_object_kill( p_intf );
     vlc_thread_join( p_intf );
+
+    module_unneed( p_intf, p_intf->p_module );
 }
 
+
+
 /* Following functions are local */
 
-/*****************************************************************************
+/**
  * RunInterface: setups necessary data and give control to the interface
- *****************************************************************************/
+ *
+ * @param p_this: interface object
+ */
 static void* RunInterface( vlc_object_t *p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
     vlc_value_t val, text;
-    char *psz_intf;
+    int canc = vlc_savecancel ();
 
     /* Variable used for interface spawning */
     var_Create( p_intf, "intf-add", VLC_VAR_STRING |
@@ -222,76 +211,32 @@ static void* RunInterface( vlc_object_t *p_this )
     var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
 
     var_AddCallback( p_intf, "intf-add", AddIntfCallback, NULL );
+    vlc_restorecancel (canc);
 
-    do
-    {
-        /* Give control to the interface */
-        if( p_intf->pf_run )
-            p_intf->pf_run( p_intf );
-        else
-        {
-            vlc_object_lock( p_intf );
-            while( vlc_object_alive( p_intf ) )
-                vlc_object_wait( p_intf );
-            vlc_object_unlock( p_intf );
-        }
+    /* Give control to the interface */
+    if( p_intf->pf_run )
+        p_intf->pf_run( p_intf );
 
-        if( !p_intf->psz_switch_intf )
-        {
-            break;
-        }
-
-        /* Make sure the old interface is completely uninitialized */
-        module_Unneed( p_intf, p_intf->p_module );
-
-        /* Provide ability to switch the main interface on the fly */
-        psz_intf = p_intf->psz_switch_intf;
-        p_intf->psz_switch_intf = NULL;
-
-        vlc_object_lock( p_intf );
-        p_intf->b_die = false; /* FIXME */
-        p_intf->b_dead = false;
-
-        vlc_object_unlock( p_intf );
-
-        p_intf->psz_intf = psz_intf;
-        p_intf->p_module = module_Need( p_intf, "interface", psz_intf, 0 );
-    }
-    while( p_intf->p_module );
     return NULL;
 }
 
 #if defined( __APPLE__ ) || defined( WIN32 )
-/*****************************************************************************
+#include "control/libvlc_internal.h" /* libvlc_InternalWait */
+/**
  * MonitorLibVLCDeath: Used when b_should_run_on_first_thread is set.
- *****************************************************************************/
+ *
+ * @param p_this: the interface object
+ */
 static void * MonitorLibVLCDeath( vlc_object_t * p_this )
 {
     intf_thread_t *p_intf = (intf_thread_t *)p_this;
     libvlc_int_t * p_libvlc = p_intf->p_libvlc;
-    vlc_object_lock( p_libvlc );
-    while(vlc_object_alive( p_libvlc ) )
-    {
-        if(p_intf->b_die)
-        {
-            vlc_object_unlock( p_libvlc );
-            return NULL;
-        }
-        vlc_object_wait( p_libvlc );
-    }
-    vlc_object_unlock( p_libvlc );
+    int canc = vlc_savecancel ();
 
-    /* Someone killed libvlc */
+    libvlc_InternalWait( p_libvlc );
 
-    /* Make sure we kill all interface objects, especially
-     * those that are blocking libvlc (running on main thread) */
-    vlc_list_t * p_list = vlc_list_find( p_libvlc, VLC_OBJECT_INTF, FIND_CHILD );
-    for( int i = 0; i < p_list->i_count; i++ )
-    {
-        vlc_object_t * p_intf = p_list->p_values[i].p_object;
-        vlc_object_kill( p_intf );
-    }
-    vlc_list_release( p_list );
+    vlc_object_kill( p_intf ); /* Kill the stupid first thread interface */
+    vlc_restorecancel (canc);
     return NULL;
 }
 #endif
@@ -299,13 +244,14 @@ static void * MonitorLibVLCDeath( vlc_object_t * p_this )
 static int AddIntfCallback( vlc_object_t *p_this, char const *psz_cmd,
                          vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
-    intf_thread_t *p_intf;
-    char *psz_intf = malloc( strlen(newval.psz_string) + sizeof(",none") );
-
     (void)psz_cmd; (void)oldval; (void)p_data;
+    intf_thread_t *p_intf;
+    char* psz_intf;
 
     /* Try to create the interface */
-    sprintf( psz_intf, "%s,none", newval.psz_string );
+    if( asprintf( &psz_intf, "%s,none", newval.psz_string ) == -1 )
+        return VLC_ENOMEM;
+
     p_intf = intf_Create( p_this->p_libvlc, psz_intf );
     free( psz_intf );
     if( p_intf == NULL )
