@@ -67,7 +67,7 @@ playlist_item_t * playlist_NodeCreate( playlist_t *p_playlist,
 
     if( !p_input )
         p_new_input = input_item_NewWithType( VLC_OBJECT(p_playlist), NULL,
-                                        psz_name, 0, NULL, -1, ITEM_TYPE_NODE );
+                                        psz_name, 0, NULL, 0, -1, ITEM_TYPE_NODE );
     p_item = playlist_ItemNewFromInput( p_playlist,
                                         p_input ? p_input : p_new_input );
     if( p_new_input )
@@ -164,7 +164,7 @@ int playlist_NodeDelete( playlist_t *p_playlist, playlist_item_t *p_root,
     else
     {
         int i;
-        var_SetInteger( p_playlist, "item-deleted", p_root->i_id );
+        var_SetInteger( p_playlist, "playlist-item-deleted", p_root->i_id );
         ARRAY_BSEARCH( p_playlist->all_items, ->i_id, int,
                        p_root->i_id, i );
         if( i != -1 )
@@ -239,35 +239,6 @@ int playlist_NodeRemoveItem( playlist_t *p_playlist,
    return VLC_SUCCESS;
 }
 
-
-/**
- * Count the children of a node
- *
- * \param p_playlist the playlist
- * \param p_node the node
- * \return the number of children
- */
-int playlist_NodeChildrenCount( playlist_t *p_playlist, playlist_item_t*p_node)
-{
-    PL_ASSERT_LOCKED;
-    int i;
-    int i_nb = 0;
-
-    if( p_node->i_children == -1 )
-        return 0;
-
-    i_nb = p_node->i_children;
-    for( i=0 ; i< p_node->i_children;i++ )
-    {
-        if( p_node->pp_children[i]->i_children == -1 )
-            break;
-        else
-            i_nb += playlist_NodeChildrenCount( p_playlist,
-                                                p_node->pp_children[i] );
-    }
-    return i_nb;
-}
-
 /**
  * Search a child of a node by its name
  *
@@ -336,7 +307,7 @@ playlist_item_t * playlist_GetPreferredNode( playlist_t *p_playlist,
     int i;
     if( p_node->p_parent == p_playlist->p_root_category )
     {
-        if( p_playlist->b_tree || p_node->p_input->b_prefers_tree )
+        if( pl_priv(p_playlist)->b_tree || p_node->p_input->b_prefers_tree )
             return p_node;
         for( i = 0 ; i< p_playlist->p_root_onelevel->i_children; i++ )
         {
@@ -347,7 +318,7 @@ playlist_item_t * playlist_GetPreferredNode( playlist_t *p_playlist,
     }
     else if( p_node->p_parent == p_playlist->p_root_onelevel )
     {
-        if( !p_playlist->b_tree || !p_node->p_input->b_prefers_tree )
+        if( !pl_priv(p_playlist)->b_tree || !p_node->p_input->b_prefers_tree )
             return p_node;
         for( i = 0 ; i< p_playlist->p_root_category->i_children; i++ )
         {
@@ -481,41 +452,40 @@ playlist_item_t *GetNextItem( playlist_t *p_playlist,
                               playlist_item_t *p_root,
                               playlist_item_t *p_item )
 {
-    playlist_item_t *p_parent;
-    int i;
+    /* If the item is NULL, return the firt child of root */
+    if( p_item == NULL )
+    {
+        if( p_root->i_children > 0 )
+            return p_root->pp_children[0];
+        else
+            return NULL;
+    }
 
     /* Node with children, get the first one */
-    if( p_item && p_item->i_children > 0 )
+    if( p_item->i_children > 0 )
         return p_item->pp_children[0];
 
-    if( p_item != NULL )
-        p_parent = p_item->p_parent;
-    else
-        p_parent = p_root;
-    for( i= 0 ; i < p_parent->i_children ; i++ )
+    playlist_item_t* p_parent = p_item->p_parent;
+    for( int i = 0 ; i < p_parent->i_children ; i++ )
     {
-        if( p_item == NULL || p_parent->pp_children[i] == p_item )
+        if( p_parent->pp_children[i] == p_item )
         {
-            if( p_item == NULL )
-                i = -1;
-
-            if( i+1 >= p_parent->i_children )
+            // Return the next children
+            if( i + 1 < p_parent->i_children )
+                return p_parent->pp_children[i+1];
+            // We are the least one, so try to have uncles
+            else
             {
-                /* Was already the last sibling. Look for uncles */
                 PL_DEBUG2( "Current item is the last of the node,"
                            "looking for uncle from %s",
                             p_parent->p_input->psz_name );
-
                 if( p_parent == p_root )
                 {
                     PL_DEBUG2( "already at root" );
                     return NULL;
                 }
-                return GetNextUncle( p_playlist, p_item, p_root );
-            }
-            else
-            {
-                return  p_parent->pp_children[i+1];
+                else
+                    return GetNextUncle( p_playlist, p_item, p_root );
             }
         }
     }
